@@ -14,7 +14,6 @@ from nautobot.core.forms import (
     ReturnURLForm,
     TableConfigForm,
 )
-from nautobot.core.forms.forms import dynamic_formset_factory
 from nautobot.core.forms.fields import (
     CommentField,
     CSVChoiceField,
@@ -41,6 +40,7 @@ from nautobot.core.forms.fields import (
     SlugField,
     TagFilterField,
 )
+from nautobot.core.forms.forms import dynamic_formset_factory
 from nautobot.core.forms.utils import (
     add_field_to_filter_form_class,
     expand_alphanumeric_pattern,
@@ -53,6 +53,7 @@ from nautobot.core.forms.utils import (
 from nautobot.core.forms.widgets import (
     APISelect,
     APISelectMultiple,
+    AutoPopulateWidget,
     BulkEditNullBooleanSelect,
     ColorSelect,
     ContentTypeSelect,
@@ -71,6 +72,7 @@ from nautobot.extras.forms import (
     CustomFieldModelBulkEditFormMixin,
     CustomFieldModelCSVForm,
     CustomFieldModelFormMixin,
+    DynamicGroupModelFormMixin,
     NautobotBulkEditForm,
     NautobotModelForm,
     NoteModelBulkEditFormMixin,
@@ -87,6 +89,7 @@ from nautobot.extras.forms.forms import (
     provider_choices_with_blank,
 )
 from nautobot.extras.forms.mixins import (
+    ContactTeamModelFilterFormMixin,
     CustomFieldModelFilterFormMixin,
     NoteFormBase,
     RelationshipModelFilterFormMixin,
@@ -96,21 +99,15 @@ from nautobot.extras.forms.mixins import (
 )
 from nautobot.ipam.formfields import IPAddressFormField, IPNetworkFormField
 
-
 __all__ = (
-    "add_blank_choice",
-    "add_field_to_filter_form_class",
-    "AddressFieldMixin",
     "APISelect",
     "APISelectMultiple",
+    "AddressFieldMixin",
+    "AutoPopulateWidget",
     "BootstrapMixin",
     "BulkEditForm",
     "BulkEditNullBooleanSelect",
     "BulkRenameForm",
-    "ColorSelect",
-    "CommentField",
-    "ConfirmationForm",
-    "ContentTypeSelect",
     "CSVChoiceField",
     "CSVContentTypeField",
     "CSVDataField",
@@ -119,34 +116,35 @@ __all__ = (
     "CSVModelForm",
     "CSVMultipleChoiceField",
     "CSVMultipleContentTypeField",
+    "ColorSelect",
+    "CommentField",
+    "ConfirmationForm",
+    "ContactTeamModelFilterFormMixin",
+    "ContentTypeSelect",
     "CustomFieldModelBulkEditFormMixin",
     "CustomFieldModelCSVForm",
     "CustomFieldModelFilterFormMixin",
     "CustomFieldModelFormMixin",
     "DatePicker",
     "DateTimePicker",
-    "dynamic_formset_factory",
     "DynamicFilterForm",
+    "DynamicGroupModelFormMixin",
     "DynamicModelChoiceField",
     "DynamicModelChoiceMixin",
     "DynamicModelMultipleChoiceField",
-    "expand_alphanumeric_pattern",
-    "expand_ipaddress_pattern",
     "ExpandableIPAddressField",
     "ExpandableNameField",
-    "form_from_model",
-    "get_git_datasource_content_choices",
-    "ImportForm",
     "IPAddressFormField",
     "IPNetworkFormField",
+    "ImportForm",
     "JSONArrayFormField",
     "JSONField",
     "LaxURLField",
     "MACAddressField",
     "MultiMatchModelMultipleChoiceField",
-    "MultipleContentTypeField",
     "MultiValueCharField",
     "MultiValueCharInput",
+    "MultipleContentTypeField",
     "NautobotBulkEditForm",
     "NautobotFilterForm",
     "NautobotModelForm",
@@ -155,15 +153,10 @@ __all__ = (
     "NoteModelFormMixin",
     "NullableDateField",
     "NumericArrayField",
-    "parse_alphanumeric_range",
-    "parse_numeric_range",
     "PrefixFieldMixin",
-    "provider_choices",
-    "provider_choices_with_blank",
     "RelationshipModelBulkEditFormMixin",
     "RelationshipModelFilterFormMixin",
     "RelationshipModelFormMixin",
-    "restrict_form_fields",
     "ReturnURLForm",
     "RoleModelBulkEditFormMixin",
     "RoleModelFilterFormMixin",
@@ -180,4 +173,16 @@ __all__ = (
     "TagFilterField",
     "TagsBulkEditFormMixin",
     "TimePicker",
+    "add_blank_choice",
+    "add_field_to_filter_form_class",
+    "dynamic_formset_factory",
+    "expand_alphanumeric_pattern",
+    "expand_ipaddress_pattern",
+    "form_from_model",
+    "get_git_datasource_content_choices",
+    "parse_alphanumeric_range",
+    "parse_numeric_range",
+    "provider_choices",
+    "provider_choices_with_blank",
+    "restrict_form_fields",
 )
