@@ -13,6 +13,7 @@ from nautobot.core.utils.data import (
     render_jinja2,
     shallow_compare_dict,
     to_meters,
+    validate_jinja2,
 )
 from nautobot.core.utils.deprecation import class_deprecated, class_deprecated_in_favor_of
 from nautobot.core.utils.filtering import (
@@ -139,5 +140,6 @@ __all__ = (
     "swap_status_initials",
     "task_queues_as_choices",
     "to_meters",
+    "validate_jinja2",
     "wrap_model_clean_methods",
 )
