@@ -4,4 +4,4 @@ from nautobot.core import checks
 # Django starts so that shared_task will use this app.
 from nautobot.core.celery import app as celery_app
 
-__all__ = ("celery_app",)
+__all__ = ("celery_app", "checks")
