@@ -1,8 +1,7 @@
-from django.contrib.admin import site as admin_site, ModelAdmin
+from constance.admin import Config, ConstanceAdmin, ConstanceForm
+from django.contrib.admin import ModelAdmin, site as admin_site
 from django.db import models
-
-from constance.admin import ConstanceAdmin, ConstanceForm, Config
-from django_celery_beat import admin  # noqa: F401
+from django_celery_beat import admin  # noqa: F401  # unused-import -- but this import installs the beat admin
 from django_celery_beat.models import (
     ClockedSchedule,
     CrontabSchedule,
@@ -10,13 +9,14 @@ from django_celery_beat.models import (
     PeriodicTask,
     SolarSchedule,
 )
+import social_django.admin  # noqa: F401  # unused-import -- but this import installs the social_django admin
 from social_django.models import Association, Nonce, UserSocialAuth
+import taggit.admin  # noqa: F401  # unused-import -- but this import installs the taggit admin
 from taggit.models import Tag
 
 from nautobot.core.forms import BootstrapMixin
 import nautobot.core.forms.widgets as widgets
 
-
 # Override default AdminSite attributes so we can avoid creating and
 # registering our own class
 admin_site.site_header = "Nautobot Administration"
