@@ -2,15 +2,18 @@ from collections import OrderedDict
 import logging
 
 from django.core.exceptions import ObjectDoesNotExist
+from django.core.validators import URLValidator
 from django.db.models import Model
 from drf_spectacular.utils import extend_schema_field
 from rest_framework import serializers
 from rest_framework.exceptions import ValidationError
+from rest_framework.fields import URLField
 from rest_framework.relations import PrimaryKeyRelatedField, RelatedField
 from timezone_field.rest_framework import TimeZoneSerializerField as TimeZoneSerializerField_
 
 from nautobot.core.api.mixins import WritableSerializerMixin
 from nautobot.core.models.utils import deconstruct_composite_key
+from nautobot.core.models.validators import EnhancedURLValidator
 from nautobot.core.utils.data import is_url, is_uuid
 from nautobot.core.utils.lookup import get_route_for_model
 
@@ -50,10 +53,10 @@ class ChoiceField(serializers.Field):
                 data = ""
         return super().validate_empty_values(data)
 
-    def to_representation(self, obj):
-        if obj == "":
+    def to_representation(self, value):
+        if value == "" and "" not in self._choices:
             return None
-        return OrderedDict([("value", obj), ("label", self._choices[obj])])
+        return OrderedDict([("value", value), ("label", self._choices[value])])
 
     def to_internal_value(self, data):
         if data == "":
@@ -120,8 +123,18 @@ class ContentTypeField(RelatedField):
             self.fail("invalid")
         return None
 
-    def to_representation(self, obj):
-        return f"{obj.app_label}.{obj.model}"
+    def to_representation(self, value):
+        return f"{value.app_label}.{value.model}"
+
+
+class LaxURLField(URLField):
+    def __init__(self, validators=None, **kwargs):
+        super().__init__(**kwargs)
+        # Discard default URLValidator added by URLField
+        self.validators = [v for v in self.validators if not isinstance(v, URLValidator)]
+        if validators is not None:
+            self.validators.extend(validators)
+        self.validators.append(EnhancedURLValidator(message=self.error_messages["invalid"]))
 
 
 @extend_schema_field(
