@@ -1,4 +1,8 @@
 from django_filters.rest_framework.backends import DjangoFilterBackend
+from rest_framework.filters import OrderingFilter
+from tree_queries.models import TreeNode
+
+from nautobot.core.api.constants import NON_FILTER_QUERY_PARAMS
 
 
 class NautobotFilterBackend(DjangoFilterBackend):
@@ -15,16 +19,19 @@ class NautobotFilterBackend(DjangoFilterBackend):
         # The default 'data' is a reference to request.GET, which is immutable; copies of this data are mutable
         data = kwargs["data"].copy()
 
-        for non_filter_param in (
-            "api_version",  # used to select the Nautobot API version
-            "depth",  # nested levels of the serializers default to depth=0
-            "format",  # "json" or "api", used in the interactive HTML REST API views
-            "include",  # used to include computed fields, relationships, config-contexts, etc. (excluded by default)
-            "limit",  # pagination
-            "offset",  # pagination
-            "sort",  # sorting of results
-        ):
+        for non_filter_param in NON_FILTER_QUERY_PARAMS:
             data.pop(non_filter_param, None)
 
         kwargs["data"] = data
         return kwargs
+
+
+class NautobotOrderingFilter(OrderingFilter):
+    """Custom Ordering Filter backend."""
+
+    def filter_queryset(self, request, queryset, view):
+        filtered_queryset = super().filter_queryset(request, queryset, view)
+        ordering = self.get_ordering(request, queryset, view)
+        if ordering and issubclass(queryset.model, TreeNode):
+            filtered_queryset = filtered_queryset.extra(order_by=ordering)
+        return filtered_queryset
