@@ -1,7 +1,6 @@
 from django.conf import settings as django_settings
 
 from nautobot.core.settings_funcs import sso_auth_enabled
-from nautobot.core.utils.navigation import is_route_new_ui_ready
 
 
 def get_saml_idp():
@@ -21,7 +20,7 @@ def get_saml_idp():
     value = ""
     if idp_map is not None:
         try:
-            idp = list(idp_map.keys())[0]
+            idp = next(iter(idp_map.keys()))
         except IndexError:
             pass
         else:
@@ -35,9 +34,6 @@ def settings(request):
     Expose Django settings in the template context. Example: {{ settings.DEBUG }}
     """
     root_template = "base_django.html"
-    if django_settings.ENABLE_ALPHA_UI:
-        use_new_ui = request.COOKIES.get("newui", False) and is_route_new_ui_ready(request.path)
-        root_template = "base_react.html" if use_new_ui else "base_django.html"
     return {
         "settings": django_settings,
         "root_template": root_template,
