@@ -1,5 +1,5 @@
 # noinspection PyUnresolvedReferences
-from django.core.management.commands.makemigrations import Command  # noqa: F401
+from django.core.management.commands.makemigrations import Command  # noqa: F401  # unused-import
 from django.db import models
 
 from nautobot.core.management import commands
