@@ -3,7 +3,6 @@ import argparse
 from django.core.management import call_command
 from django.core.management.base import BaseCommand
 
-
 HELP_TEXT = """
 Performs Nautobot common server upgrade operations using a single entrypoint.
 
@@ -11,7 +10,6 @@ This will run the following management commands with default settings, in order:
 
 - migrate
 - trace_paths
-- build_ui --npm-install
 - collectstatic
 - remove_stale_contenttypes
 - clearsessions
@@ -29,13 +27,6 @@ class Command(BaseCommand):
         return parser
 
     def add_arguments(self, parser):
-        parser.add_argument(
-            "--build-ui",
-            action="store_true",
-            dest="build_ui",
-            default=False,
-            help="Build the user interface.",
-        )
         parser.add_argument(
             "--no-clearsessions",
             action="store_false",
@@ -99,6 +90,7 @@ class Command(BaseCommand):
             self.stdout.write("Performing database migrations...")
             call_command(
                 "migrate",
+                skip_checks=False,  # make sure Postgres version check and others are applied
                 interactive=False,
                 traceback=options["traceback"],
                 verbosity=options["verbosity"],
@@ -111,12 +103,6 @@ class Command(BaseCommand):
             call_command("trace_paths", no_input=True)
             self.stdout.write()
 
-        # Run build
-        if options.get("build_ui"):
-            self.stdout.write("Building user interface...")
-            call_command("build_ui", npm_install=True)
-            self.stdout.write()
-
         # Run collectstatic
         if options.get("collectstatic"):
             self.stdout.write("Collecting static files...")
