@@ -1,9 +1,12 @@
+from zoneinfo import ZoneInfo
+
 from django.conf import settings
 from django.contrib.auth.middleware import RemoteUserMiddleware as RemoteUserMiddleware_
 from django.db import ProgrammingError
 from django.http import Http404
 from django.urls import resolve
 from django.urls.exceptions import Resolver404
+from django.utils import timezone
 from django.utils.deprecation import MiddlewareMixin
 
 from nautobot.core.api.utils import is_api_request, rest_api_server_error
@@ -12,9 +15,9 @@ from nautobot.core.authentication import (
     assign_permissions_to_user,
 )
 from nautobot.core.settings_funcs import (
-    sso_auth_enabled,
-    remote_auth_enabled,
     ldap_auth_enabled,
+    remote_auth_enabled,
+    sso_auth_enabled,
 )
 from nautobot.core.views import server_error
 from nautobot.extras.choices import ObjectChangeEventContextChoices
@@ -149,3 +152,16 @@ class ExceptionHandlingMiddleware:
             return server_error(request, template_name=custom_template)
 
         return None
+
+
+class UserDefinedTimeZoneMiddleware:
+    def __init__(self, get_response):
+        self.get_response = get_response
+
+    def __call__(self, request):
+        if request.user.is_authenticated:
+            if tzname := request.user.get_config("timezone"):
+                timezone.activate(ZoneInfo(tzname))
+            else:
+                timezone.deactivate()
+        return self.get_response(request)
