@@ -1,24 +1,27 @@
-import socket
 from collections import OrderedDict
+import socket
 
 from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import F
-from django.http import HttpResponseForbidden, HttpResponse
+from django.http import HttpResponse, HttpResponseForbidden
 from django.shortcuts import get_object_or_404
 from django.views.decorators.clickjacking import xframe_options_sameorigin
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import extend_schema, OpenApiParameter
 from rest_framework.decorators import action
 from rest_framework.mixins import ListModelMixin
+from rest_framework.parsers import JSONParser, MultiPartParser
 from rest_framework.permissions import IsAuthenticated
 from rest_framework.response import Response
-from rest_framework.routers import APIRootView
 from rest_framework.viewsets import GenericViewSet, ViewSet
 
 from nautobot.circuits.models import Circuit
+from nautobot.cloud.models import CloudAccount
 from nautobot.core.api.exceptions import ServiceUnavailable
+from nautobot.core.api.parsers import NautobotCSVParser
 from nautobot.core.api.utils import get_serializer_for_model
+from nautobot.core.api.views import ModelViewSet
 from nautobot.core.models.querysets import count_related
 from nautobot.dcim import filters
 from nautobot.dcim.models import (
@@ -28,21 +31,31 @@ from nautobot.dcim.models import (
     ConsolePortTemplate,
     ConsoleServerPort,
     ConsoleServerPortTemplate,
+    Controller,
+    ControllerManagedDeviceGroup,
     Device,
     DeviceBay,
     DeviceBayTemplate,
+    DeviceFamily,
     DeviceRedundancyGroup,
     DeviceType,
+    DeviceTypeToSoftwareImageFile,
     FrontPort,
     FrontPortTemplate,
     Interface,
     InterfaceRedundancyGroup,
     InterfaceRedundancyGroupAssociation,
     InterfaceTemplate,
+    InterfaceVDCAssignment,
+    InventoryItem,
     Location,
     LocationType,
     Manufacturer,
-    InventoryItem,
+    Module,
+    ModuleBay,
+    ModuleBayTemplate,
+    ModuleFamily,
+    ModuleType,
     Platform,
     PowerFeed,
     PowerOutlet,
@@ -55,29 +68,24 @@ from nautobot.dcim.models import (
     RackReservation,
     RearPort,
     RearPortTemplate,
+    SoftwareImageFile,
+    SoftwareVersion,
     VirtualChassis,
+    VirtualDeviceContext,
 )
 from nautobot.extras.api.views import (
     ConfigContextQuerySetMixin,
+    CustomFieldModelViewSet,
     NautobotModelViewSet,
 )
 from nautobot.extras.choices import SecretsGroupAccessTypeChoices, SecretsGroupSecretTypeChoices
 from nautobot.extras.secrets.exceptions import SecretError
-from nautobot.ipam.models import Prefix, VLAN
+from nautobot.ipam.models import IPAddress, Prefix, VLAN
 from nautobot.virtualization.models import VirtualMachine
+
 from . import serializers
 from .exceptions import MissingFilterException
 
-
-class DCIMRootView(APIRootView):
-    """
-    DCIM API root view
-    """
-
-    def get_view_name(self):
-        return "DCIM"
-
-
 # Mixins
 
 
@@ -136,7 +144,7 @@ class PassThroughPortMixin:
 
 
 class LocationTypeViewSet(NautobotModelViewSet):
-    queryset = LocationType.objects.select_related("parent").prefetch_related("content_types")
+    queryset = LocationType.objects.all()
     serializer_class = serializers.LocationTypeSerializer
     filterset_class = filters.LocationTypeFilterSet
 
@@ -147,17 +155,13 @@ class LocationTypeViewSet(NautobotModelViewSet):
 
 
 class LocationViewSet(NautobotModelViewSet):
-    queryset = (
-        Location.objects.select_related("location_type", "parent", "status", "tenant")
-        .prefetch_related("tags")
-        .annotate(
-            device_count=count_related(Device, "location"),
-            rack_count=count_related(Rack, "location"),
-            prefix_count=count_related(Prefix, "location"),
-            vlan_count=count_related(VLAN, "location"),
-            circuit_count=count_related(Circuit, "circuit_terminations__location"),
-            virtual_machine_count=count_related(VirtualMachine, "cluster__location"),
-        )
+    queryset = Location.objects.annotate(
+        device_count=count_related(Device, "location"),
+        rack_count=count_related(Rack, "location"),
+        prefix_count=count_related(Prefix, "locations"),
+        vlan_count=count_related(VLAN, "locations"),
+        circuit_count=count_related(Circuit, "circuit_terminations__location"),
+        virtual_machine_count=count_related(VirtualMachine, "cluster__location"),
     )
     serializer_class = serializers.LocationSerializer
     filterset_class = filters.LocationFilterSet
@@ -169,7 +173,7 @@ class LocationViewSet(NautobotModelViewSet):
 
 
 class RackGroupViewSet(NautobotModelViewSet):
-    queryset = RackGroup.objects.annotate(rack_count=count_related(Rack, "rack_group")).select_related("location")
+    queryset = RackGroup.objects.annotate(rack_count=count_related(Rack, "rack_group"))
     serializer_class = serializers.RackGroupSerializer
     filterset_class = filters.RackGroupFilterSet
 
@@ -180,13 +184,9 @@ class RackGroupViewSet(NautobotModelViewSet):
 
 
 class RackViewSet(NautobotModelViewSet):
-    queryset = (
-        Rack.objects.select_related("location", "rack_group__location", "status", "role", "tenant")
-        .prefetch_related("tags")
-        .annotate(
-            device_count=count_related(Device, "rack"),
-            power_feed_count=count_related(PowerFeed, "rack"),
-        )
+    queryset = Rack.objects.select_related("role", "status", "rack_group__location").annotate(
+        device_count=count_related(Device, "rack"),
+        power_feed_count=count_related(PowerFeed, "rack"),
     )
     serializer_class = serializers.RackSerializer
     filterset_class = filters.RackFilterSet
@@ -229,6 +229,8 @@ class RackViewSet(NautobotModelViewSet):
                 exclude=data["exclude"],
                 expand_devices=data["expand_devices"],
             )
+            if data["is_occupied"] is not None:
+                elevation = [u for u in elevation if u["occupied"] == data["is_occupied"]]
 
             # Enable filtering rack units by ID
             q = data["q"]
@@ -249,7 +251,7 @@ class RackViewSet(NautobotModelViewSet):
 
 
 class RackReservationViewSet(NautobotModelViewSet):
-    queryset = RackReservation.objects.select_related("rack", "user", "tenant")
+    queryset = RackReservation.objects.all()
     serializer_class = serializers.RackReservationSerializer
     filterset_class = filters.RackReservationFilterSet
 
@@ -261,6 +263,7 @@ class RackReservationViewSet(NautobotModelViewSet):
 
 class ManufacturerViewSet(NautobotModelViewSet):
     queryset = Manufacturer.objects.annotate(
+        cloud_account_count=count_related(CloudAccount, "provider"),
         device_type_count=count_related(DeviceType, "manufacturer"),
         inventory_item_count=count_related(InventoryItem, "manufacturer"),
         platform_count=count_related(Platform, "manufacturer"),
@@ -269,19 +272,29 @@ class ManufacturerViewSet(NautobotModelViewSet):
     filterset_class = filters.ManufacturerFilterSet
 
 
+#
+# Device Family
+#
+
+
+class DeviceFamilyViewSet(NautobotModelViewSet):
+    queryset = DeviceFamily.objects.annotate(
+        device_type_count=count_related(DeviceType, "device_family"),
+    )
+    serializer_class = serializers.DeviceFamilySerializer
+    filterset_class = filters.DeviceFamilyFilterSet
+
+
 #
 # Device types
 #
 
 
 class DeviceTypeViewSet(NautobotModelViewSet):
-    queryset = (
-        DeviceType.objects.select_related("manufacturer")
-        .prefetch_related("tags")
-        .annotate(device_count=count_related(Device, "device_type"))
-    )
+    queryset = DeviceType.objects.annotate(device_count=count_related(Device, "device_type"))
     serializer_class = serializers.DeviceTypeSerializer
     filterset_class = filters.DeviceTypeFilterSet
+    parser_classes = [JSONParser, NautobotCSVParser, MultiPartParser]
 
 
 #
@@ -289,54 +302,62 @@ class DeviceTypeViewSet(NautobotModelViewSet):
 #
 
 
-class ConsolePortTemplateViewSet(NautobotModelViewSet):
-    queryset = ConsolePortTemplate.objects.select_related("device_type__manufacturer")
+class ConsolePortTemplateViewSet(CustomFieldModelViewSet):
+    queryset = ConsolePortTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
     serializer_class = serializers.ConsolePortTemplateSerializer
     filterset_class = filters.ConsolePortTemplateFilterSet
 
 
-class ConsoleServerPortTemplateViewSet(NautobotModelViewSet):
-    queryset = ConsoleServerPortTemplate.objects.select_related("device_type__manufacturer")
+class ConsoleServerPortTemplateViewSet(CustomFieldModelViewSet):
+    queryset = ConsoleServerPortTemplate.objects.select_related(
+        "device_type__manufacturer", "module_type__manufacturer"
+    )
     serializer_class = serializers.ConsoleServerPortTemplateSerializer
     filterset_class = filters.ConsoleServerPortTemplateFilterSet
 
 
-class PowerPortTemplateViewSet(NautobotModelViewSet):
-    queryset = PowerPortTemplate.objects.select_related("device_type__manufacturer")
+class PowerPortTemplateViewSet(CustomFieldModelViewSet):
+    queryset = PowerPortTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
     serializer_class = serializers.PowerPortTemplateSerializer
     filterset_class = filters.PowerPortTemplateFilterSet
 
 
-class PowerOutletTemplateViewSet(NautobotModelViewSet):
-    queryset = PowerOutletTemplate.objects.select_related("device_type__manufacturer")
+class PowerOutletTemplateViewSet(CustomFieldModelViewSet):
+    queryset = PowerOutletTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
     serializer_class = serializers.PowerOutletTemplateSerializer
     filterset_class = filters.PowerOutletTemplateFilterSet
 
 
-class InterfaceTemplateViewSet(NautobotModelViewSet):
-    queryset = InterfaceTemplate.objects.select_related("device_type__manufacturer")
+class InterfaceTemplateViewSet(CustomFieldModelViewSet):
+    queryset = InterfaceTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
     serializer_class = serializers.InterfaceTemplateSerializer
     filterset_class = filters.InterfaceTemplateFilterSet
 
 
-class FrontPortTemplateViewSet(NautobotModelViewSet):
-    queryset = FrontPortTemplate.objects.select_related("device_type__manufacturer")
+class FrontPortTemplateViewSet(CustomFieldModelViewSet):
+    queryset = FrontPortTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
     serializer_class = serializers.FrontPortTemplateSerializer
     filterset_class = filters.FrontPortTemplateFilterSet
 
 
-class RearPortTemplateViewSet(NautobotModelViewSet):
-    queryset = RearPortTemplate.objects.select_related("device_type__manufacturer")
+class RearPortTemplateViewSet(CustomFieldModelViewSet):
+    queryset = RearPortTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
     serializer_class = serializers.RearPortTemplateSerializer
     filterset_class = filters.RearPortTemplateFilterSet
 
 
-class DeviceBayTemplateViewSet(NautobotModelViewSet):
+class DeviceBayTemplateViewSet(CustomFieldModelViewSet):
     queryset = DeviceBayTemplate.objects.select_related("device_type__manufacturer")
     serializer_class = serializers.DeviceBayTemplateSerializer
     filterset_class = filters.DeviceBayTemplateFilterSet
 
 
+class ModuleBayTemplateViewSet(CustomFieldModelViewSet):
+    queryset = ModuleBayTemplate.objects.select_related("device_type__manufacturer", "module_type__manufacturer")
+    serializer_class = serializers.ModuleBayTemplateSerializer
+    filterset_class = filters.ModuleBayTemplateFilterSet
+
+
 #
 # Platforms
 #
@@ -359,19 +380,8 @@ class PlatformViewSet(NautobotModelViewSet):
 class DeviceViewSet(ConfigContextQuerySetMixin, NautobotModelViewSet):
     queryset = Device.objects.select_related(
         "device_type__manufacturer",
-        "role",
-        "tenant",
-        "platform",
-        "rack",
-        "location",
-        "parent_bay",
-        "primary_ip4",
-        "primary_ip6",
         "virtual_chassis__master",
-        "device_redundancy_group",
-        "secrets_group",
-        "status",
-    ).prefetch_related("tags", "primary_ip4__nat_outside_list", "primary_ip6__nat_outside_list")
+    ).prefetch_related("primary_ip4__nat_outside_list", "primary_ip6__nat_outside_list")
     serializer_class = serializers.DeviceSerializer
     filterset_class = filters.DeviceFilterSet
 
@@ -468,9 +478,9 @@ class DeviceViewSet(ConfigContextQuerySetMixin, NautobotModelViewSet):
         # Get NAPALM enable-secret from the device if present
         if device.secrets_group:
             # Work around inconsistent enable password arg in NAPALM drivers
-            enable_password_arg = "secret"
+            enable_password_arg = "secret"  # noqa: S105  # hardcoded-password-string -- false positive
             if device.platform.napalm_driver.lower() == "eos":
-                enable_password_arg = "enable_password"
+                enable_password_arg = "enable_password"  # noqa: S105  # hardcoded-password-string -- false positive
             try:
                 optional_args[enable_password_arg] = device.secrets_group.get_secret_value(
                     SecretsGroupAccessTypeChoices.TYPE_GENERIC,
@@ -534,77 +544,69 @@ class DeviceViewSet(ConfigContextQuerySetMixin, NautobotModelViewSet):
 
 
 class ConsolePortViewSet(PathEndpointMixin, NautobotModelViewSet):
-    queryset = ConsolePort.objects.select_related("device", "cable").prefetch_related(
-        "_path__destination", "_cable_peer", "tags"
-    )
+    queryset = ConsolePort.objects.prefetch_related("_path__destination", "_cable_peer")
     serializer_class = serializers.ConsolePortSerializer
     filterset_class = filters.ConsolePortFilterSet
 
 
 class ConsoleServerPortViewSet(PathEndpointMixin, NautobotModelViewSet):
-    queryset = ConsoleServerPort.objects.select_related("device", "cable").prefetch_related(
-        "_path__destination", "_cable_peer", "tags"
-    )
+    queryset = ConsoleServerPort.objects.prefetch_related("_path__destination", "_cable_peer")
     serializer_class = serializers.ConsoleServerPortSerializer
     filterset_class = filters.ConsoleServerPortFilterSet
 
 
 class PowerPortViewSet(PathEndpointMixin, NautobotModelViewSet):
-    queryset = PowerPort.objects.select_related("device", "cable").prefetch_related(
-        "_path__destination", "_cable_peer", "tags"
-    )
+    queryset = PowerPort.objects.prefetch_related("_path__destination", "_cable_peer")
     serializer_class = serializers.PowerPortSerializer
     filterset_class = filters.PowerPortFilterSet
 
 
 class PowerOutletViewSet(PathEndpointMixin, NautobotModelViewSet):
-    queryset = PowerOutlet.objects.select_related("device", "cable").prefetch_related(
-        "_path__destination", "_cable_peer", "tags"
-    )
+    queryset = PowerOutlet.objects.prefetch_related("_path__destination", "_cable_peer")
     serializer_class = serializers.PowerOutletSerializer
     filterset_class = filters.PowerOutletFilterSet
 
 
 class InterfaceViewSet(PathEndpointMixin, NautobotModelViewSet):
-    queryset = Interface.objects.select_related(
-        "device",
-        "parent_interface",
-        "bridge",
-        "lag",
-        "status",
-        "cable",
-        "untagged_vlan",
-    ).prefetch_related("tags", "_path__destination", "_cable_peer", "ip_addresses", "tagged_vlans")
+    queryset = Interface.objects.prefetch_related("_path__destination", "_cable_peer").annotate(
+        _ip_address_count=count_related(IPAddress, "interfaces")  # avoid conflict with Interface.ip_address_count()
+    )
     serializer_class = serializers.InterfaceSerializer
     filterset_class = filters.InterfaceFilterSet
 
 
 class FrontPortViewSet(PassThroughPortMixin, NautobotModelViewSet):
-    queryset = FrontPort.objects.select_related(
-        "device__device_type__manufacturer", "rear_port", "cable"
-    ).prefetch_related("tags")
+    queryset = FrontPort.objects.select_related("device__device_type__manufacturer")
     serializer_class = serializers.FrontPortSerializer
     filterset_class = filters.FrontPortFilterSet
 
 
 class RearPortViewSet(PassThroughPortMixin, NautobotModelViewSet):
-    queryset = RearPort.objects.select_related("device__device_type__manufacturer", "cable").prefetch_related("tags")
+    queryset = RearPort.objects.select_related("device__device_type__manufacturer")
     serializer_class = serializers.RearPortSerializer
     filterset_class = filters.RearPortFilterSet
 
 
 class DeviceBayViewSet(NautobotModelViewSet):
-    queryset = DeviceBay.objects.select_related("installed_device").prefetch_related("tags")
+    queryset = DeviceBay.objects.all()
     serializer_class = serializers.DeviceBaySerializer
     filterset_class = filters.DeviceBayFilterSet
 
 
 class InventoryItemViewSet(NautobotModelViewSet):
-    queryset = InventoryItem.objects.select_related("device", "manufacturer").prefetch_related("tags")
+    queryset = InventoryItem.objects.all()
     serializer_class = serializers.InventoryItemSerializer
     filterset_class = filters.InventoryItemFilterSet
 
 
+class ModuleBayViewSet(NautobotModelViewSet):
+    queryset = ModuleBay.objects.select_related("parent_device__tenant", "parent_device__location").prefetch_related(
+        "installed_module"
+    )
+    serializer_class = serializers.ModuleBaySerializer
+    filterset_class = filters.ModuleBayFilterSet
+
+
 #
 # Connections
 # TODO: remove these in favor of using the ConsolePort/PowerPort/Interface API endpoints and/or Cable endpoint.
@@ -639,10 +641,18 @@ class InterfaceConnectionViewSet(ListModelMixin, GenericViewSet):
 
 
 class CableViewSet(NautobotModelViewSet):
-    queryset = Cable.objects.select_related("status").prefetch_related("termination_a", "termination_b")
+    queryset = Cable.objects.prefetch_related("termination_a", "termination_b")
     serializer_class = serializers.CableSerializer
     filterset_class = filters.CableFilterSet
 
+    def get_queryset(self):
+        # 6933 fix: with prefetch related in queryset
+        # DeviceInterface is not properly cleared of _path_id
+        queryset = super().get_queryset()
+        if self.action == "destroy":
+            queryset = queryset.prefetch_related(None)
+        return queryset
+
 
 #
 # Virtual chassis
@@ -650,9 +660,7 @@ class CableViewSet(NautobotModelViewSet):
 
 
 class VirtualChassisViewSet(NautobotModelViewSet):
-    queryset = VirtualChassis.objects.prefetch_related("tags").annotate(
-        member_count=count_related(Device, "virtual_chassis")
-    )
+    queryset = VirtualChassis.objects.annotate(member_count=count_related(Device, "virtual_chassis"))
     serializer_class = serializers.VirtualChassisSerializer
     filterset_class = filters.VirtualChassisFilterSet
 
@@ -663,9 +671,7 @@ class VirtualChassisViewSet(NautobotModelViewSet):
 
 
 class PowerPanelViewSet(NautobotModelViewSet):
-    queryset = PowerPanel.objects.select_related("location", "rack_group").annotate(
-        power_feed_count=count_related(PowerFeed, "power_panel")
-    )
+    queryset = PowerPanel.objects.annotate(power_feed_count=count_related(PowerFeed, "power_panel"))
     serializer_class = serializers.PowerPanelSerializer
     filterset_class = filters.PowerPanelFilterSet
 
@@ -676,12 +682,7 @@ class PowerPanelViewSet(NautobotModelViewSet):
 
 
 class PowerFeedViewSet(PathEndpointMixin, NautobotModelViewSet):
-    queryset = PowerFeed.objects.select_related(
-        "power_panel",
-        "rack",
-        "cable",
-        "status",
-    ).prefetch_related("tags", "_cable_peer", "_path__destination")
+    queryset = PowerFeed.objects.prefetch_related("_cable_peer", "_path__destination")
     serializer_class = serializers.PowerFeedSerializer
     filterset_class = filters.PowerFeedFilterSet
 
@@ -692,7 +693,7 @@ class PowerFeedViewSet(PathEndpointMixin, NautobotModelViewSet):
 
 
 class DeviceRedundancyGroupViewSet(NautobotModelViewSet):
-    queryset = DeviceRedundancyGroup.objects.select_related("status").prefetch_related("devices")
+    queryset = DeviceRedundancyGroup.objects.all()
     serializer_class = serializers.DeviceRedundancyGroupSerializer
     filterset_class = filters.DeviceRedundancyGroupFilterSet
 
@@ -703,7 +704,7 @@ class DeviceRedundancyGroupViewSet(NautobotModelViewSet):
 
 
 class InterfaceRedundancyGroupViewSet(NautobotModelViewSet):
-    queryset = InterfaceRedundancyGroup.objects.select_related("status").prefetch_related("interfaces")
+    queryset = InterfaceRedundancyGroup.objects.all()
     serializer_class = serializers.InterfaceRedundancyGroupSerializer
     filterset_class = filters.InterfaceRedundancyGroupFilterSet
 
@@ -761,7 +762,7 @@ class ConnectedDeviceViewSet(ViewSet):
 
         # Determine local interface from peer interface's connection
         peer_interface = get_object_or_404(
-            Interface.objects.all(),
+            Interface.objects.restrict(request.user, "view"),
             device__name=peer_device_name,
             name=peer_interface_name,
         )
@@ -771,3 +772,87 @@ class ConnectedDeviceViewSet(ViewSet):
             return Response()
 
         return Response(serializers.DeviceSerializer(local_interface.device, context={"request": request}).data)
+
+
+#
+# Software image files
+#
+
+
+class SoftwareImageFileViewSet(NautobotModelViewSet):
+    queryset = SoftwareImageFile.objects.all()
+    serializer_class = serializers.SoftwareImageFileSerializer
+    filterset_class = filters.SoftwareImageFileFilterSet
+
+
+class SoftwareVersionViewSet(NautobotModelViewSet):
+    queryset = SoftwareVersion.objects.all()
+    serializer_class = serializers.SoftwareVersionSerializer
+    filterset_class = filters.SoftwareVersionFilterSet
+
+
+class DeviceTypeToSoftwareImageFileViewSet(ModelViewSet):
+    queryset = DeviceTypeToSoftwareImageFile.objects.all()
+    serializer_class = serializers.DeviceTypeToSoftwareImageFileSerializer
+    filterset_class = filters.DeviceTypeToSoftwareImageFileFilterSet
+
+
+#
+# Controllers
+#
+
+
+class ControllerViewSet(NautobotModelViewSet):
+    queryset = Controller.objects.all()
+    serializer_class = serializers.ControllerSerializer
+    filterset_class = filters.ControllerFilterSet
+
+
+class ControllerManagedDeviceGroupViewSet(NautobotModelViewSet):
+    queryset = ControllerManagedDeviceGroup.objects.all()
+    serializer_class = serializers.ControllerManagedDeviceGroupSerializer
+    filterset_class = filters.ControllerManagedDeviceGroupFilterSet
+
+
+#
+# Modules
+#
+
+
+class ModuleViewSet(NautobotModelViewSet):
+    queryset = Module.objects.select_related(
+        "parent_module_bay__parent_device__location",
+        "parent_module_bay__parent_device__tenant",
+        "module_type__manufacturer",
+    )
+    serializer_class = serializers.ModuleSerializer
+    filterset_class = filters.ModuleFilterSet
+
+
+class ModuleTypeViewSet(NautobotModelViewSet):
+    queryset = ModuleType.objects.all()
+    serializer_class = serializers.ModuleTypeSerializer
+    filterset_class = filters.ModuleTypeFilterSet
+
+
+class VirtualDeviceContextViewSet(NautobotModelViewSet):
+    queryset = VirtualDeviceContext.objects.all()
+    serializer_class = serializers.VirtualDeviceContextSerializer
+    filterset_class = filters.VirtualDeviceContextFilterSet
+
+
+class InterfaceVDCAssignmentViewSet(ModelViewSet):
+    queryset = InterfaceVDCAssignment.objects.all()
+    serializer_class = serializers.InterfaceVDCAssignmentSerializer
+    filterset_class = filters.InterfaceVDCAssignmentFilterSet
+
+
+class ModuleFamilyViewSet(NautobotModelViewSet):
+    """API viewset for interacting with ModuleFamily objects."""
+
+    queryset = ModuleFamily.objects.annotate(
+        module_type_count=count_related(ModuleType, "module_family"),
+        module_bay_count=count_related(ModuleBay, "module_family"),
+    )
+    serializer_class = serializers.ModuleFamilySerializer
+    filterset_class = filters.ModuleFamilyFilterSet
