@@ -1,3 +1,5 @@
+from decimal import Decimal
+
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.core.validators import MaxValueValidator, MinValueValidator
@@ -252,8 +254,8 @@ class PowerPortTemplate(ModularComponentTemplateModel):
     power_factor = models.DecimalField(
         max_digits=4,
         decimal_places=2,
-        default="0.95",
-        validators=[MinValueValidator(0.01), MaxValueValidator(1.00)],
+        default=Decimal("0.95"),
+        validators=[MinValueValidator(Decimal("0.01")), MaxValueValidator(Decimal("1.00"))],
         help_text="Power factor (0.01-1.00) for converting between watts (W) and volt-amps (VA). Defaults to 0.95.",
     )
 
