@@ -1,15 +1,10 @@
 from nautobot.core.apps import (
-    NavContext,
-    NavGrouping,
-    NavItem,
     NavMenuAddButton,
     NavMenuGroup,
     NavMenuItem,
-    NavMenuImportButton,
     NavMenuTab,
 )
 
-
 menu_items = (
     NavMenuTab(
         name="Organization",
@@ -19,32 +14,10 @@ menu_items = (
                 name="Locations",
                 weight=150,
                 items=(
-                    NavMenuItem(
-                        link="dcim:locationtype_list",
-                        name="Location Types",
-                        weight=100,
-                        permissions=[
-                            "dcim.view_locationtype",
-                        ],
-                        buttons=(
-                            NavMenuAddButton(
-                                link="dcim:locationtype_add",
-                                permissions=[
-                                    "dcim.add_locationtype",
-                                ],
-                            ),
-                            NavMenuImportButton(
-                                link="dcim:locationtype_import",
-                                permissions=[
-                                    "dcim.add_locationtype",
-                                ],
-                            ),
-                        ),
-                    ),
                     NavMenuItem(
                         link="dcim:location_list",
                         name="Locations",
-                        weight=200,
+                        weight=100,
                         permissions=[
                             "dcim.view_location",
                         ],
@@ -55,10 +28,20 @@ menu_items = (
                                     "dcim.add_location",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:location_import",
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:locationtype_list",
+                        name="Location Types",
+                        weight=200,
+                        permissions=[
+                            "dcim.view_locationtype",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:locationtype_add",
                                 permissions=[
-                                    "dcim.add_location",
+                                    "dcim.add_locationtype",
                                 ],
                             ),
                         ),
@@ -83,12 +66,6 @@ menu_items = (
                                     "dcim.add_rack",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:rack_import",
-                                permissions=[
-                                    "dcim.add_rack",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
@@ -105,17 +82,11 @@ menu_items = (
                                     "dcim.add_rackgroup",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:rackgroup_import",
-                                permissions=[
-                                    "dcim.add_rackgroup",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
                         link="dcim:rackreservation_list",
-                        name="Reservations",
+                        name="Rack Reservations",
                         weight=400,
                         permissions=[
                             "dcim.view_rackreservation",
@@ -127,12 +98,6 @@ menu_items = (
                                     "dcim.add_rackreservation",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:rackreservation_import",
-                                permissions=[
-                                    "dcim.add_rackreservation",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
@@ -170,34 +135,6 @@ menu_items = (
                                     "dcim.add_device",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:device_import",
-                                permissions=[
-                                    "dcim.add_device",
-                                ],
-                            ),
-                        ),
-                    ),
-                    NavMenuItem(
-                        link="dcim:platform_list",
-                        name="Platforms",
-                        weight=300,
-                        permissions=[
-                            "dcim.view_platform",
-                        ],
-                        buttons=(
-                            NavMenuAddButton(
-                                link="dcim:platform_add",
-                                permissions=[
-                                    "dcim.add_platform",
-                                ],
-                            ),
-                            NavMenuImportButton(
-                                link="dcim:platform_import",
-                                permissions=[
-                                    "dcim.add_platform",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
@@ -214,12 +151,6 @@ menu_items = (
                                     "dcim.add_virtualchassis",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:virtualchassis_import",
-                                permissions=[
-                                    "dcim.add_virtualchassis",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
@@ -236,12 +167,6 @@ menu_items = (
                                     "dcim.add_deviceredundancygroup",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:deviceredundancygroup_import",
-                                permissions=[
-                                    "dcim.add_deviceredundancygroup",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
@@ -258,10 +183,20 @@ menu_items = (
                                     "dcim.add_interfaceredundancygroup",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:interfaceredundancygroup_import",
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:virtualdevicecontext_list",
+                        name="Virtual Device Contexts",
+                        weight=700,
+                        permissions=[
+                            "dcim.view_virtualdevicecontext",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:virtualdevicecontext_add",
                                 permissions=[
-                                    "dcim.add_interfaceredundancygroup",
+                                    "dcim.add_virtualdevicecontext",
                                 ],
                             ),
                         ),
@@ -286,10 +221,20 @@ menu_items = (
                                     "dcim.add_devicetype",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:devicetype_import",
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:devicefamily_list",
+                        name="Device Families",
+                        weight=200,
+                        permissions=[
+                            "dcim.view_devicefamily",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:devicefamily_add",
                                 permissions=[
-                                    "dcim.add_devicetype",
+                                    "dcim.add_devicefamily",
                                 ],
                             ),
                         ),
@@ -297,7 +242,7 @@ menu_items = (
                     NavMenuItem(
                         link="dcim:manufacturer_list",
                         name="Manufacturers",
-                        weight=200,
+                        weight=300,
                         permissions=[
                             "dcim.view_manufacturer",
                         ],
@@ -308,10 +253,150 @@ menu_items = (
                                     "dcim.add_manufacturer",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:manufacturer_import",
+                        ),
+                    ),
+                ),
+            ),
+            NavMenuGroup(
+                name="Modules",
+                weight=250,
+                items=(
+                    NavMenuItem(
+                        link="dcim:module_list",
+                        name="Modules",
+                        weight=100,
+                        permissions=[
+                            "dcim.view_module",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:module_add",
                                 permissions=[
-                                    "dcim.add_manufacturer",
+                                    "dcim.add_module",
+                                ],
+                            ),
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:moduletype_list",
+                        name="Module Types",
+                        weight=200,
+                        permissions=[
+                            "dcim.view_moduletype",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:moduletype_add",
+                                permissions=[
+                                    "dcim.add_moduletype",
+                                ],
+                            ),
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:modulefamily_list",
+                        name="Module Families",
+                        weight=300,
+                        permissions=[
+                            "dcim.view_modulefamily",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:modulefamily_add",
+                                permissions=[
+                                    "dcim.add_modulefamily",
+                                ],
+                            ),
+                        ),
+                    ),
+                ),
+            ),
+            NavMenuGroup(
+                name="Software",
+                weight=300,
+                items=(
+                    NavMenuItem(
+                        link="dcim:platform_list",
+                        name="Platforms",
+                        weight=100,
+                        permissions=[
+                            "dcim.view_platform",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:platform_add",
+                                permissions=[
+                                    "dcim.add_platform",
+                                ],
+                            ),
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:softwareversion_list",
+                        name="Software Versions",
+                        weight=200,
+                        permissions=[
+                            "dcim.view_softwareversion",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:softwareversion_add",
+                                permissions=[
+                                    "dcim.add_softwareversion",
+                                ],
+                            ),
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:softwareimagefile_list",
+                        name="Software Image Files",
+                        weight=300,
+                        permissions=[
+                            "dcim.view_softwareimagefile",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:softwareimagefile_add",
+                                permissions=[
+                                    "dcim.add_softwareimagefile",
+                                ],
+                            ),
+                        ),
+                    ),
+                ),
+            ),
+            NavMenuGroup(
+                name="Controllers",
+                weight=400,
+                items=(
+                    NavMenuItem(
+                        link="dcim:controller_list",
+                        name="Controllers",
+                        weight=100,
+                        permissions=[
+                            "dcim.view_controller",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:controller_add",
+                                permissions=[
+                                    "dcim.add_controller",
+                                ],
+                            ),
+                        ),
+                    ),
+                    NavMenuItem(
+                        link="dcim:controllermanageddevicegroup_list",
+                        name="Device Groups",
+                        weight=200,
+                        permissions=[
+                            "dcim.view_controllermanageddevicegroup",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:controllermanageddevicegroup_add",
+                                permissions=[
+                                    "dcim.add_controllermanageddevicegroup",
                                 ],
                             ),
                         ),
@@ -320,7 +405,7 @@ menu_items = (
             ),
             NavMenuGroup(
                 name="Connections",
-                weight=300,
+                weight=500,
                 items=(
                     NavMenuItem(
                         link="dcim:cable_list",
@@ -329,14 +414,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_cable",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:cable_import",
-                                permissions=[
-                                    "dcim.add_cable",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:console_connections_list",
@@ -371,7 +449,7 @@ menu_items = (
             ),
             NavMenuGroup(
                 name="Device Components",
-                weight=400,
+                weight=600,
                 items=(
                     NavMenuItem(
                         link="dcim:interface_list",
@@ -380,14 +458,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_interface",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:interface_import",
-                                permissions=[
-                                    "dcim.add_interface",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:frontport_list",
@@ -396,14 +467,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_frontport",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:frontport_import",
-                                permissions=[
-                                    "dcim.add_frontport",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:rearport_list",
@@ -412,14 +476,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_rearport",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:rearport_import",
-                                permissions=[
-                                    "dcim.add_rearport",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:consoleport_list",
@@ -428,14 +485,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_consoleport",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:consoleport_import",
-                                permissions=[
-                                    "dcim.add_consoleport",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:consoleserverport_list",
@@ -444,14 +494,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_consoleserverport",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:consoleserverport_import",
-                                permissions=[
-                                    "dcim.add_consoleserverport",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:powerport_list",
@@ -460,14 +503,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_powerport",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:powerport_import",
-                                permissions=[
-                                    "dcim.add_powerport",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:poweroutlet_list",
@@ -476,14 +512,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_poweroutlet",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:poweroutlet_import",
-                                permissions=[
-                                    "dcim.add_poweroutlet",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:devicebay_list",
@@ -492,14 +521,16 @@ menu_items = (
                         permissions=[
                             "dcim.view_devicebay",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:devicebay_import",
-                                permissions=[
-                                    "dcim.add_devicebay",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
+                    ),
+                    NavMenuItem(
+                        link="dcim:modulebay_list",
+                        name="Module Bays",
+                        weight=850,
+                        permissions=[
+                            "dcim.view_modulebay",
+                        ],
+                        buttons=(),
                     ),
                     NavMenuItem(
                         link="dcim:inventoryitem_list",
@@ -508,14 +539,7 @@ menu_items = (
                         permissions=[
                             "dcim.view_inventoryitem",
                         ],
-                        buttons=(
-                            NavMenuImportButton(
-                                link="dcim:inventoryitem_import",
-                                permissions=[
-                                    "dcim.add_inventoryitem",
-                                ],
-                            ),
-                        ),
+                        buttons=(),
                     ),
                 ),
             ),
@@ -542,12 +566,6 @@ menu_items = (
                                     "dcim.add_powerfeed",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:powerfeed_import",
-                                permissions=[
-                                    "dcim.add_powerfeed",
-                                ],
-                            ),
                         ),
                     ),
                     NavMenuItem(
@@ -563,12 +581,6 @@ menu_items = (
                                     "dcim.add_powerpanel",
                                 ],
                             ),
-                            NavMenuImportButton(
-                                link="dcim:powerpanel_import",
-                                permissions=[
-                                    "dcim.add_powerpanel",
-                                ],
-                            ),
                         ),
                     ),
                 ),
@@ -576,223 +588,3 @@ menu_items = (
         ),
     ),
 )
-
-
-navigation = (
-    NavContext(
-        name="Inventory",
-        groups=(
-            NavGrouping(
-                name="Devices",
-                weight=100,
-                items=(
-                    NavItem(
-                        name="Devices",
-                        link="dcim:device_list",
-                        weight=100,
-                        permissions=["dcim.view_device"],
-                    ),
-                    NavItem(
-                        name="Device Types",
-                        weight=200,
-                        link="dcim:devicetype_list",
-                        permissions=["dcim.view_devicetype"],
-                    ),
-                    NavItem(
-                        link="dcim:platform_list",
-                        name="Platforms",
-                        weight=300,
-                        permissions=[
-                            "dcim.view_platform",
-                        ],
-                    ),
-                    NavItem(
-                        link="dcim:manufacturer_list",
-                        name="Manufacturers",
-                        weight=400,
-                        permissions=[
-                            "dcim.view_manufacturer",
-                        ],
-                    ),
-                    NavItem(
-                        link="dcim:virtualchassis_list",
-                        name="Virtual Chassis",
-                        weight=500,
-                        permissions=[
-                            "dcim.view_virtualchassis",
-                        ],
-                    ),
-                    NavItem(
-                        name="Device Redundancy Groups",
-                        weight=600,
-                        link="dcim:deviceredundancygroup_list",
-                        permissions=["dcim.view_deviceredundancygroup"],
-                    ),
-                    NavGrouping(
-                        name="Connections",
-                        weight=700,
-                        items=(
-                            NavItem(
-                                name="Cables",
-                                weight=100,
-                                link="dcim:cable_list",
-                                permissions=["dcim.view_cable"],
-                            ),
-                            NavItem(
-                                name="Console Connections",
-                                weight=200,
-                                link="dcim:console_connections_list",
-                                permissions=[
-                                    "dcim.view_consoleport",
-                                    "dcim.view_consoleserverport",
-                                ],
-                            ),
-                            NavItem(
-                                name="Power Connections",
-                                weight=300,
-                                link="dcim:power_connections_list",
-                                permissions=[
-                                    "dcim.view_powerport",
-                                    "dcim.view_poweroutlet",
-                                ],
-                            ),
-                            NavItem(
-                                name="Interface Connections",
-                                weight=400,
-                                link="dcim:interface_connections_list",
-                                permissions=["dcim.view_interface"],
-                            ),
-                        ),
-                    ),
-                    NavGrouping(
-                        name="Components",
-                        weight=800,
-                        items=(
-                            NavItem(
-                                name="Interfaces",
-                                weight=100,
-                                link="dcim:interface_list",
-                                permissions=["dcim.view_interface"],
-                            ),
-                            NavItem(
-                                name="Front Ports",
-                                weight=200,
-                                link="dcim:frontport_list",
-                                permissions=["dcim.view_frontport"],
-                            ),
-                            NavItem(
-                                name="Rear Ports",
-                                weight=300,
-                                link="dcim:rearport_list",
-                                permissions=["dcim.view_rearport"],
-                            ),
-                            NavItem(
-                                name="Console Ports",
-                                weight=400,
-                                link="dcim:consoleport_list",
-                                permissions=["dcim.view_consoleport"],
-                            ),
-                            NavItem(
-                                name="Console Server Ports",
-                                weight=500,
-                                link="dcim:consoleserverport_list",
-                                permissions=["dcim.view_consoleserverport"],
-                            ),
-                            NavItem(
-                                name="Power Ports",
-                                weight=600,
-                                link="dcim:powerport_list",
-                                permissions=["dcim.view_powerport"],
-                            ),
-                            NavItem(
-                                name="Power Outlets",
-                                weight=700,
-                                link="dcim:poweroutlet_list",
-                                permissions=["dcim.view_poweroutlet"],
-                            ),
-                            NavItem(
-                                name="Device Bays",
-                                weight=800,
-                                link="dcim:devicebay_list",
-                                permissions=["dcim.view_devicebay"],
-                            ),
-                            NavItem(
-                                name="Inventory Items",
-                                weight=900,
-                                link="dcim:inventoryitem_list",
-                                permissions=["dcim.view_inventoryitem"],
-                            ),
-                        ),
-                    ),
-                    # space reserved for Dynamic Groups item with weight 900
-                    NavItem(
-                        name="Racks",
-                        weight=1000,
-                        link="dcim:rack_list",
-                        permissions=["dcim.view_rack"],
-                    ),
-                    NavItem(
-                        name="Rack Groups",
-                        weight=1100,
-                        link="dcim:rackgroup_list",
-                        permissions=["dcim.view_rackgroup"],
-                    ),
-                    NavItem(
-                        name="Rack Reservations",
-                        weight=1200,
-                        link="dcim:rackreservation_list",
-                        permissions=["dcim.view_rackreservation"],
-                    ),
-                    NavItem(
-                        name="Rack Elevations",
-                        weight=1300,
-                        link="dcim:rack_elevation_list",
-                        permissions=["dcim.view_rack"],
-                    ),
-                ),
-            ),
-            NavGrouping(
-                name="Organization",
-                weight=200,
-                items=(
-                    NavItem(
-                        link="dcim:location_list",
-                        name="Locations",
-                        weight=100,
-                        permissions=[
-                            "dcim.view_location",
-                        ],
-                    ),
-                    NavItem(
-                        link="dcim:locationtype_list",
-                        name="Location Types",
-                        weight=200,
-                        permissions=[
-                            "dcim.view_locationtype",
-                        ],
-                    ),
-                ),
-            ),
-            # space reserved for Tenants at weight 300
-            # space reserved for Circuits at weight 400
-            NavGrouping(
-                name="Power",
-                weight=500,
-                items=(
-                    NavItem(
-                        name="Power Feeds",
-                        link="dcim:powerfeed_list",
-                        weight=100,
-                        permissions=["dcim.view_powerfeed"],
-                    ),
-                    NavItem(
-                        name="Power Panels",
-                        weight=200,
-                        link="dcim:powerpanel_list",
-                        permissions=["dcim.view_powerpanel"],
-                    ),
-                ),
-            ),
-        ),
-    ),
-)
