@@ -12,10 +12,13 @@ from nautobot.dcim.models import (
     ConsolePortTemplate,
     ConsoleServerPortTemplate,
     DeviceBayTemplate,
+    DeviceFamily,
     DeviceType,
     FrontPortTemplate,
     InterfaceTemplate,
     Manufacturer,
+    ModuleBayTemplate,
+    ModuleType,
     PowerOutletTemplate,
     PowerPortTemplate,
     RearPortTemplate,
@@ -25,10 +28,13 @@ __all__ = (
     "ConsolePortTemplateTable",
     "ConsoleServerPortTemplateTable",
     "DeviceBayTemplateTable",
+    "DeviceFamilyTable",
     "DeviceTypeTable",
     "FrontPortTemplateTable",
     "InterfaceTemplateTable",
     "ManufacturerTable",
+    "ModuleBayTemplateTable",
+    "ModuleTypeTable",
     "PowerOutletTemplateTable",
     "PowerPortTemplateTable",
     "RearPortTemplateTable",
@@ -43,9 +49,18 @@ __all__ = (
 class ManufacturerTable(BaseTable):
     pk = ToggleColumn()
     name = tables.LinkColumn()
-    device_type_count = tables.Column(verbose_name="Device Types")
-    inventory_item_count = tables.Column(verbose_name="Inventory Items")
-    platform_count = tables.Column(verbose_name="Platforms")
+    cloud_account_count = LinkedCountColumn(
+        viewname="cloud:cloudaccount_list", url_params={"provider": "name"}, verbose_name="Cloud Accounts"
+    )
+    device_type_count = LinkedCountColumn(
+        viewname="dcim:devicetype_list", url_params={"manufacturer": "name"}, verbose_name="Device Types"
+    )
+    inventory_item_count = LinkedCountColumn(
+        viewname="dcim:inventoryitem_list", url_params={"manufacturer": "name"}, verbose_name="Inventory Items"
+    )
+    platform_count = LinkedCountColumn(
+        viewname="dcim:platform_list", url_params={"manufacturer": "name"}, verbose_name="Platforms"
+    )
     actions = ButtonsColumn(Manufacturer)
 
     class Meta(BaseTable.Meta):
@@ -53,6 +68,7 @@ class ManufacturerTable(BaseTable):
         fields = (
             "pk",
             "name",
+            "cloud_account_count",
             "device_type_count",
             "inventory_item_count",
             "platform_count",
@@ -61,6 +77,32 @@ class ManufacturerTable(BaseTable):
         )
 
 
+#
+# Device Family
+#
+
+
+class DeviceFamilyTable(BaseTable):
+    pk = ToggleColumn()
+    name = tables.Column(linkify=True)
+    device_type_count = LinkedCountColumn(
+        viewname="dcim:devicetype_list", url_params={"device_family": "name"}, verbose_name="Device Types"
+    )
+    actions = ButtonsColumn(DeviceFamily)
+    tags = TagColumn(url_name="dcim:devicefamily_list")
+
+    class Meta(BaseTable.Meta):
+        model = DeviceFamily
+        fields = (
+            "pk",
+            "name",
+            "device_type_count",
+            "description",
+            "actions",
+            "tags",
+        )
+
+
 #
 # Device types
 #
@@ -68,7 +110,9 @@ class ManufacturerTable(BaseTable):
 
 class DeviceTypeTable(BaseTable):
     pk = ToggleColumn()
+    manufacturer = tables.Column(linkify=True)
     model = tables.Column(linkify=True, verbose_name="Device Type")
+    device_family = tables.Column(linkify=True)
     is_full_depth = BooleanColumn(verbose_name="Full Depth")
     device_count = LinkedCountColumn(
         viewname="dcim:device_list",
@@ -76,6 +120,7 @@ class DeviceTypeTable(BaseTable):
         verbose_name="Devices",
     )
     tags = TagColumn(url_name="dcim:devicetype_list")
+    actions = ButtonsColumn(DeviceType)
 
     class Meta(BaseTable.Meta):
         model = DeviceType
@@ -83,12 +128,14 @@ class DeviceTypeTable(BaseTable):
             "pk",
             "model",
             "manufacturer",
+            "device_family",
             "part_number",
             "u_height",
             "is_full_depth",
             "subdevice_role",
             "device_count",
             "tags",
+            "actions",
         )
         default_columns = (
             "pk",
@@ -98,6 +145,45 @@ class DeviceTypeTable(BaseTable):
             "u_height",
             "is_full_depth",
             "device_count",
+            "actions",
+        )
+
+
+#
+# Module types
+#
+
+
+class ModuleTypeTable(BaseTable):
+    pk = ToggleColumn()
+    manufacturer = tables.Column(linkify=True)
+    model = tables.Column(linkify=True, verbose_name="Module Type")
+    module_family = tables.Column(linkify=True, verbose_name="Family")
+    module_count = LinkedCountColumn(
+        viewname="dcim:module_list",
+        url_params={"module_type": "pk"},
+        verbose_name="Modules",
+    )
+    tags = TagColumn(url_name="dcim:moduletype_list")
+
+    class Meta(BaseTable.Meta):
+        model = ModuleType
+        fields = (
+            "pk",
+            "model",
+            "manufacturer",
+            "part_number",
+            "module_family",
+            "module_count",
+            "tags",
+        )
+        default_columns = (
+            "pk",
+            "model",
+            "manufacturer",
+            "part_number",
+            "module_family",
+            "module_count",
         )
 
 
@@ -115,7 +201,7 @@ class ConsolePortTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=ConsolePortTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_consoleports",
+        return_url_extra=r"%3Ftab=consoleports",
     )
 
     class Meta(BaseTable.Meta):
@@ -128,7 +214,7 @@ class ConsoleServerPortTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=ConsoleServerPortTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_consoleserverports",
+        return_url_extra=r"%3Ftab=consoleserverports",
     )
 
     class Meta(BaseTable.Meta):
@@ -141,7 +227,7 @@ class PowerPortTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=PowerPortTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_powerports",
+        return_url_extra=r"%3Ftab=powerports",
     )
 
     class Meta(BaseTable.Meta):
@@ -153,6 +239,7 @@ class PowerPortTemplateTable(ComponentTemplateTable):
             "type",
             "maximum_draw",
             "allocated_draw",
+            "power_factor",
             "description",
             "actions",
         )
@@ -163,7 +250,7 @@ class PowerOutletTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=PowerOutletTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_poweroutlets",
+        return_url_extra=r"%3Ftab=poweroutlets",
     )
 
     class Meta(BaseTable.Meta):
@@ -173,7 +260,7 @@ class PowerOutletTemplateTable(ComponentTemplateTable):
             "name",
             "label",
             "type",
-            "power_port",
+            "power_port_template",
             "feed_leg",
             "description",
             "actions",
@@ -186,7 +273,7 @@ class InterfaceTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=InterfaceTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_interfaces",
+        return_url_extra=r"%3Ftab=interfaces",
     )
 
     class Meta(BaseTable.Meta):
@@ -200,7 +287,7 @@ class FrontPortTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=FrontPortTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_frontports",
+        return_url_extra=r"%3Ftab=frontports",
     )
 
     class Meta(BaseTable.Meta):
@@ -210,7 +297,7 @@ class FrontPortTemplateTable(ComponentTemplateTable):
             "name",
             "label",
             "type",
-            "rear_port",
+            "rear_port_template",
             "rear_port_position",
             "description",
             "actions",
@@ -222,7 +309,7 @@ class RearPortTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=RearPortTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_rearports",
+        return_url_extra=r"%3Ftab=rearports",
     )
 
     class Meta(BaseTable.Meta):
@@ -235,10 +322,54 @@ class DeviceBayTemplateTable(ComponentTemplateTable):
     actions = ButtonsColumn(
         model=DeviceBayTemplate,
         buttons=("edit", "delete"),
-        return_url_extra="%23tab_devicebays",
+        return_url_extra=r"%3Ftab=devicebays",
     )
 
     class Meta(BaseTable.Meta):
         model = DeviceBayTemplate
         fields = ("pk", "name", "label", "description", "actions")
         empty_text = "None"
+
+
+class ModuleBayTemplateTable(ComponentTemplateTable):
+    actions = ButtonsColumn(
+        model=ModuleBayTemplate,
+        buttons=("edit", "delete"),
+        return_url_extra=r"%3Ftab=modulebays",
+    )
+    module_family = tables.Column(verbose_name="Family", linkify=True)
+    requires_first_party_modules = BooleanColumn(verbose_name="Requires First-Party Modules")
+
+    class Meta(BaseTable.Meta):
+        model = ModuleBayTemplate
+        fields = (
+            "pk",
+            "name",
+            "position",
+            "module_family",
+            "label",
+            "requires_first_party_modules",
+            "description",
+            "actions",
+        )
+        default_columns = (
+            "pk",
+            "name",
+            "position",
+            "module_family",
+            "label",
+            "requires_first_party_modules",
+            "description",
+            "actions",
+        )
+        field_order = (
+            "pk",
+            "name",
+            "position",
+            "module_family",
+            "label",
+            "requires_first_party_modules",
+            "description",
+            "actions",
+        )
+        empty_text = "None"
