@@ -1,23 +1,13 @@
+from copy import deepcopy
 import uuid
 
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
-from netutils.lib_mapper import (
-    ANSIBLE_LIB_MAPPER_REVERSE,
-    HIERCONFIG_LIB_MAPPER_REVERSE,
-    NAPALM_LIB_MAPPER_REVERSE,
-    NETMIKO_LIB_MAPPER_REVERSE,
-    NETUTILSPARSER_LIB_MAPPER_REVERSE,
-    NTCTEMPLATES_LIB_MAPPER_REVERSE,
-    PYATS_LIB_MAPPER_REVERSE,
-    PYNTC_LIB_MAPPER_REVERSE,
-    SCRAPLI_LIB_MAPPER_REVERSE,
-)
+from netutils.lib_mapper import NAME_TO_ALL_LIB_MAPPER, NAME_TO_LIB_MAPPER_REVERSE
 
+from nautobot.core.choices import ColorChoices
 from nautobot.core.utils.config import get_settings_or_config
-from nautobot.core.utils.color import hex_to_rgb, lighten_color, rgb_to_hex
 from nautobot.dcim.choices import InterfaceModeChoices
-from nautobot.dcim.constants import NETUTILS_NETWORK_DRIVER_MAPPING_NAMES
 
 
 def compile_path_node(ct_id, object_id):
@@ -55,11 +45,14 @@ def cable_status_color_css(record):
     """
     if not record.cable:
         return ""
-    # The status colors are for use with labels and such, and tend to be quite bright.
-    # For this function we want a much milder, mellower color suitable as a row background.
-    base_color = record.cable.get_status_color().strip("#")
-    lighter_color = rgb_to_hex(*lighten_color(*hex_to_rgb(base_color), 0.75))
-    return f"background-color: #{lighter_color}"
+    else:
+        CABLE_STATUS_TO_CSS_CLASS = {
+            ColorChoices.COLOR_GREEN: "success",
+            ColorChoices.COLOR_AMBER: "warning",
+            ColorChoices.COLOR_CYAN: "info",
+        }
+        status_color = record.cable.get_status_color().strip("#")
+        return CABLE_STATUS_TO_CSS_CLASS.get(status_color, "")
 
 
 def get_network_driver_mapping_tool_names():
@@ -68,9 +61,8 @@ def get_network_driver_mapping_tool_names():
 
     Tool names are "ansible", "hier_config", "napalm", "netmiko", etc...
     """
-    network_driver_names = NETUTILS_NETWORK_DRIVER_MAPPING_NAMES.copy()
-    network_driver_names.update(get_settings_or_config("NETWORK_DRIVERS").keys())
-
+    network_driver_names = set(NAME_TO_LIB_MAPPER_REVERSE.keys())
+    network_driver_names.update(get_settings_or_config("NETWORK_DRIVERS", fallback={}).keys())
     return sorted(network_driver_names)
 
 
@@ -91,26 +83,10 @@ def get_all_network_driver_mappings():
             etc...
         }
     """
-    network_driver_mappings = {}
-
-    # initialize mapping from netutils library
-    for tool_name, mappings in (
-        ("ansible", ANSIBLE_LIB_MAPPER_REVERSE),
-        ("hier_config", HIERCONFIG_LIB_MAPPER_REVERSE),
-        ("napalm", NAPALM_LIB_MAPPER_REVERSE),
-        ("netmiko", NETMIKO_LIB_MAPPER_REVERSE),
-        ("netutils_parser", NETUTILSPARSER_LIB_MAPPER_REVERSE),
-        ("ntc_templates", NTCTEMPLATES_LIB_MAPPER_REVERSE),
-        ("pyats", PYATS_LIB_MAPPER_REVERSE),
-        ("pyntc", PYNTC_LIB_MAPPER_REVERSE),
-        ("scrapli", SCRAPLI_LIB_MAPPER_REVERSE),
-    ):
-        for normalized_name, mapped_name in mappings.items():
-            network_driver_mappings.setdefault(normalized_name, {})
-            network_driver_mappings[normalized_name][tool_name] = mapped_name
+    network_driver_mappings = deepcopy(NAME_TO_ALL_LIB_MAPPER)
 
     # add mappings from optional NETWORK_DRIVERS setting
-    network_drivers_config = get_settings_or_config("NETWORK_DRIVERS")
+    network_drivers_config = get_settings_or_config("NETWORK_DRIVERS", fallback={})
     for tool_name, mappings in network_drivers_config.items():
         for normalized_name, mapped_name in mappings.items():
             network_driver_mappings.setdefault(normalized_name, {})
@@ -136,11 +112,14 @@ def validate_interface_tagged_vlans(instance, model, pk_set):
         )
 
     # Filter the model objects based on the primary keys passed in kwargs and exclude the ones that have
-    # a location that is not the parent's location or None
-    # TODO: after Location model replaced Site, which was not a hierarchical model, should we allow users to add a VLAN
-    # belongs to the parent Location or the child location of the parent device to the `tagged_vlan` field of the interface?
+    # a location that is not the parent's location, or parent's location's ancestors, or None
+    location = getattr(instance.parent, "location", None)
+    if location:
+        location_ids = location.ancestors(include_self=True).values_list("id", flat=True)
+    else:
+        location_ids = []
     tagged_vlans = (
-        model.objects.filter(pk__in=pk_set).exclude(location__isnull=True).exclude(location=instance.parent.location)
+        model.objects.filter(pk__in=pk_set).exclude(locations__isnull=True).exclude(locations__in=location_ids)
     )
 
     if tagged_vlans.count():
@@ -148,7 +127,17 @@ def validate_interface_tagged_vlans(instance, model, pk_set):
             {
                 "tagged_vlans": (
                     f"Tagged VLAN with names {list(tagged_vlans.values_list('name', flat=True))} must all belong to the "
-                    f"same location as the interface's parent device, or it must be global."
+                    "same location as the interface's parent device, "
+                    "one of the parent locations of the interface's parent device's location, or it must be global."
                 )
             }
         )
+
+
+def convert_watts_to_va(watts, power_factor):
+    """
+    Convert watts to VA using power factor.
+    """
+    if not watts:
+        return 0
+    return int(watts / power_factor)
