@@ -5,6 +5,8 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from drf_spectacular.utils import extend_schema_field
 from rest_framework import serializers
+from rest_framework.validators import UniqueTogetherValidator
+from timezone_field.rest_framework import TimeZoneSerializerField
 
 from nautobot.core.api import (
     BaseModelSerializer,
@@ -23,6 +25,7 @@ from nautobot.core.api.utils import (
     nested_serializers_for_models,
     return_nested_serializer_data_based_on_depth,
 )
+from nautobot.core.constants import CHARFIELD_MAX_LENGTH
 from nautobot.core.models.utils import get_all_concrete_models
 from nautobot.dcim.api.serializers import (
     DeviceSerializer,
@@ -30,6 +33,9 @@ from nautobot.dcim.api.serializers import (
     RackSerializer,
 )
 from nautobot.extras import choices, models
+from nautobot.extras.api.mixins import (
+    TaggedModelSerializerMixin,
+)
 from nautobot.extras.choices import (
     CustomFieldFilterLogicChoices,
     CustomFieldTypeChoices,
@@ -37,20 +43,21 @@ from nautobot.extras.choices import (
     JobResultStatusChoices,
     ObjectChangeActionChoices,
 )
-from nautobot.extras.api.mixins import (
-    TaggedModelSerializerMixin,
-)
 from nautobot.extras.datasources import get_datasource_content_choices
 from nautobot.extras.models import (
     ComputedField,
     ConfigContext,
     ConfigContextSchema,
+    Contact,
+    ContactAssociation,
     CustomField,
     CustomFieldChoice,
     CustomLink,
     DynamicGroup,
     DynamicGroupMembership,
     ExportTemplate,
+    ExternalIntegration,
+    FileProxy,
     GitRepository,
     GraphQLQuery,
     ImageAttachment,
@@ -58,22 +65,36 @@ from nautobot.extras.models import (
     JobButton,
     JobHook,
     JobLogEntry,
+    JobQueue,
+    JobQueueAssignment,
     JobResult,
+    MetadataChoice,
+    MetadataType,
     Note,
     ObjectChange,
+    ObjectMetadata,
     Relationship,
     RelationshipAssociation,
     Role,
+    SavedView,
     ScheduledJob,
     Secret,
     SecretsGroup,
     SecretsGroupAssociation,
+    StaticGroupAssociation,
     Status,
     Tag,
+    Team,
+    UserSavedViewAssociation,
     Webhook,
 )
 from nautobot.extras.models.mixins import NotesMixin
-from nautobot.extras.utils import ChangeLoggedModelsQuery, FeatureQuery, RoleModelsQuery, TaggableClassesQuery
+from nautobot.extras.utils import (
+    ChangeLoggedModelsQuery,
+    FeatureQuery,
+    RoleModelsQuery,
+    TaggableClassesQuery,
+)
 
 from .fields import MultipleChoiceJSONField
 
@@ -114,12 +135,12 @@ class ConfigContextSerializer(ValidatedModelSerializer, TaggedModelSerializerMix
     owner = serializers.SerializerMethodField(read_only=True)
 
     # Conditional enablement of dynamic groups filtering
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-
+    @property
+    def fields(self):
+        fields = super().fields
         if not settings.CONFIG_CONTEXT_DYNAMIC_GROUPS_ENABLED:
-            # In the case of a nested serializer, we won't have a `dynamic_groups` field at all.
-            self.fields.pop("dynamic_groups", None)
+            fields.pop("dynamic_groups", None)
+        return fields
 
     class Meta:
         model = ConfigContext
@@ -173,12 +194,88 @@ class ConfigContextSchemaSerializer(NautobotModelSerializer):
         return return_nested_serializer_data_based_on_depth(self, depth, obj, obj.owner, "owner")
 
 
+#
+# Contacts
+#
+
+
+class ContactSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = Contact
+        fields = "__all__"
+        # https://www.django-rest-framework.org/api-guide/validators/#optional-fields
+        validators = []
+        extra_kwargs = {
+            "email": {"default": ""},
+            "phone": {"default": ""},
+            "teams": {"required": False},
+        }
+
+    def get_field_names(self, declared_fields, info):
+        """Add reverse M2M for teams to the fields for this serializer."""
+        field_names = list(super().get_field_names(declared_fields, info))
+        self.extend_field_names(field_names, "teams")
+        return field_names
+
+    def validate(self, attrs):
+        local_attrs = attrs.copy()
+        local_attrs.pop("teams", None)
+        validator = UniqueTogetherValidator(queryset=Contact.objects.all(), fields=("name", "phone", "email"))
+        validator(local_attrs, self)
+        super().validate(local_attrs)
+        return attrs
+
+
+class ContactAssociationSerializer(NautobotModelSerializer):
+    associated_object_type = ContentTypeField(queryset=ContentType.objects.all(), many=False)
+
+    class Meta:
+        model = ContactAssociation
+        fields = "__all__"
+        validators = []
+        extra_kwargs = {
+            "contact": {"required": False},
+            "team": {"required": False},
+        }
+
+    def validate(self, attrs):
+        # Validate uniqueness of (associated object, associated object type, contact/team, role)
+        unique_together_fields = None
+
+        if attrs.get("contact") and attrs.get("role"):
+            unique_together_fields = (
+                "associated_object_type",
+                "associated_object_id",
+                "contact",
+                "role",
+            )
+        elif attrs.get("team") and attrs.get("role"):
+            unique_together_fields = (
+                "associated_object_type",
+                "associated_object_id",
+                "team",
+                "role",
+            )
+
+        if unique_together_fields is not None:
+            validator = UniqueTogetherValidator(
+                queryset=ContactAssociation.objects.all(),
+                fields=unique_together_fields,
+            )
+            validator(attrs, self)
+
+        super().validate(attrs)
+
+        return attrs
+
+
 #
 # ContentTypes
 #
 
 
 class ContentTypeSerializer(BaseModelSerializer):
+    id = serializers.IntegerField(read_only=True)
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:contenttype-detail")
     display = serializers.SerializerMethodField()
 
@@ -187,8 +284,8 @@ class ContentTypeSerializer(BaseModelSerializer):
         fields = "__all__"
 
     @extend_schema_field(serializers.CharField)
-    def get_display(self, obj):
-        return obj.app_labeled_name
+    def get_display(self, instance):
+        return instance.app_labeled_name
 
 
 #
@@ -242,7 +339,7 @@ class DynamicGroupMembershipSerializer(ValidatedModelSerializer):
         fields = "__all__"
 
 
-class DynamicGroupSerializer(NautobotModelSerializer):
+class DynamicGroupSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("dynamic_groups").get_query()).order_by("app_label", "model"),
     )
@@ -252,10 +349,52 @@ class DynamicGroupSerializer(NautobotModelSerializer):
         fields = "__all__"
         extra_kwargs = {
             "children": {"source": "dynamic_group_memberships", "read_only": True},
-            "filter": {"read_only": False},
+            "filter": {"read_only": False, "required": False},
         }
 
 
+class SavedViewSerializer(ValidatedModelSerializer):
+    class Meta:
+        model = SavedView
+        fields = "__all__"
+
+
+class UserSavedViewAssociationSerializer(ValidatedModelSerializer):
+    class Meta:
+        model = UserSavedViewAssociation
+        fields = "__all__"
+        validators = []
+
+
+class StaticGroupAssociationSerializer(NautobotModelSerializer):
+    associated_object_type = ContentTypeField(
+        queryset=ContentType.objects.filter(FeatureQuery("dynamic_groups").get_query()).order_by("app_label", "model"),
+    )
+    associated_object = serializers.SerializerMethodField(read_only=True)
+
+    class Meta:
+        model = StaticGroupAssociation
+        fields = "__all__"
+
+    @extend_schema_field(
+        PolymorphicProxySerializer(
+            component_name="DynamicGroupAssociatedObject",
+            resource_type_field_name="object_type",
+            serializers=lambda: nested_serializers_for_models(FeatureQuery("dynamic_groups").list_subclasses()),
+        )
+    )
+    def get_associated_object(self, obj):
+        if obj.associated_object is None:
+            return None
+        try:
+            depth = get_nested_serializer_depth(self)
+            return return_nested_serializer_data_based_on_depth(
+                self, depth, obj, obj.associated_object, "associated_object"
+            )
+        except SerializerNotFound:
+            return None
+
+
 #
 # Export templates
 #
@@ -293,12 +432,34 @@ class ExportTemplateSerializer(RelationshipModelSerializerMixin, ValidatedModelS
         return return_nested_serializer_data_based_on_depth(self, depth, obj, obj.owner, "owner")
 
 
+#
+# External integrations
+#
+
+
+class ExternalIntegrationSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = ExternalIntegration
+        fields = "__all__"
+
+
+#
+# File proxies
+#
+
+
+class FileProxySerializer(BaseModelSerializer):
+    class Meta:
+        model = FileProxy
+        exclude = ["file"]
+
+
 #
 # Git repositories
 #
 
 
-class GitRepositorySerializer(NautobotModelSerializer):
+class GitRepositorySerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
     """Git repositories defined as a data source."""
 
     provided_contents = MultipleChoiceJSONField(
@@ -318,12 +479,33 @@ class GitRepositorySerializer(NautobotModelSerializer):
 
 
 class GraphQLQuerySerializer(ValidatedModelSerializer, NotesSerializerMixin):
-    variables = serializers.DictField(required=False, allow_null=True, default={})
+    variables = serializers.DictField(read_only=True)
+    owner_content_type = ContentTypeField(
+        queryset=ContentType.objects.filter(FeatureQuery("graphql_query_owners").get_query()),
+        required=False,
+        allow_null=True,
+        default=None,
+    )
+    owner = serializers.SerializerMethodField(read_only=True)
 
     class Meta:
         model = GraphQLQuery
         fields = "__all__"
 
+    @extend_schema_field(
+        PolymorphicProxySerializer(
+            component_name="GraphQLQueryOwner",
+            resource_type_field_name="object_type",
+            serializers=lambda: nested_serializers_for_models(FeatureQuery("graphql_query_owners").list_subclasses()),
+            allow_null=True,
+        )
+    )
+    def get_owner(self, obj):
+        if obj.owner is None:
+            return None
+        depth = get_nested_serializer_depth(self)
+        return return_nested_serializer_data_based_on_depth(self, depth, obj, obj.owner, "owner")
+
 
 class GraphQLQueryInputSerializer(serializers.Serializer):
     variables = serializers.DictField(allow_null=True, default={})
@@ -345,17 +527,17 @@ class ImageAttachmentSerializer(ValidatedModelSerializer):
         model = ImageAttachment
         fields = "__all__"
 
-    def validate(self, data):
+    def validate(self, attrs):
         # Validate that the parent object exists
         try:
-            data["content_type"].get_object_for_this_type(id=data["object_id"])
+            attrs["content_type"].get_object_for_this_type(id=attrs["object_id"])
         except ObjectDoesNotExist:
-            raise serializers.ValidationError(f"Invalid parent object: {data['content_type']} ID {data['object_id']}")
+            raise serializers.ValidationError(f"Invalid parent object: {attrs['content_type']} ID {attrs['object_id']}")
 
         # Enforce model validation
-        super().validate(data)
+        super().validate(attrs)
 
-        return data
+        return attrs
 
     @extend_schema_field(
         PolymorphicProxySerializer(
@@ -379,28 +561,44 @@ class ImageAttachmentSerializer(ValidatedModelSerializer):
 
 
 class JobSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+    # task_queues and task_queues_override are added to maintain backward compatibility with versions pre v2.4.
+    task_queues = serializers.JSONField(read_only=True, required=False)
+    task_queues_override = serializers.BooleanField(read_only=True, required=False)
+
     class Meta:
         model = Job
         fields = "__all__"
 
-    def validate(self, data):
+    def validate(self, attrs):
         # note no validation for on creation of jobs because we do not support user creation of Job records via API
         if self.instance:
-            has_sensitive_variables = data.get("has_sensitive_variables", self.instance.has_sensitive_variables)
-            approval_required = data.get("approval_required", self.instance.approval_required)
+            has_sensitive_variables = attrs.get("has_sensitive_variables", self.instance.has_sensitive_variables)
+            approval_required = attrs.get("approval_required", self.instance.approval_required)
 
             if approval_required and has_sensitive_variables:
                 error_message = "A job with sensitive variables cannot also be marked as requiring approval"
                 errors = {}
 
-                if "approval_required" in data:
+                if "approval_required" in attrs:
                     errors["approval_required"] = [error_message]
-                if "has_sensitive_variables" in data:
+                if "has_sensitive_variables" in attrs:
                     errors["has_sensitive_variables"] = [error_message]
 
                 raise serializers.ValidationError(errors)
 
-        return super().validate(data)
+        return super().validate(attrs)
+
+
+class JobQueueSerializer(NautobotModelSerializer, TaggedModelSerializerMixin):
+    class Meta:
+        model = JobQueue
+        fields = "__all__"
+
+
+class JobQueueAssignmentSerializer(ValidatedModelSerializer):
+    class Meta:
+        model = JobQueueAssignment
+        fields = "__all__"
 
 
 class JobVariableSerializer(serializers.Serializer):
@@ -427,6 +625,9 @@ class JobVariableSerializer(serializers.Serializer):
 
 class ScheduledJobSerializer(BaseModelSerializer):
     # start_time = serializers.DateTimeField(format=None, required=False)
+    # queue is added to maintain backward compatibility with versions pre v2.4.
+    queue = serializers.CharField(read_only=True, required=False)
+    time_zone = TimeZoneSerializerField(required=False)
 
     class Meta:
         model = ScheduledJob
@@ -444,6 +645,15 @@ class JobResultSerializer(CustomFieldModelSerializerMixin, BaseModelSerializer):
     class Meta:
         model = JobResult
         fields = "__all__"
+        extra_kwargs = {
+            "files": {"read_only": True},
+        }
+
+    def get_field_names(self, declared_fields, info):
+        """Add reverse relation to related FileProxy objects."""
+        fields = list(super().get_field_names(declared_fields, info))
+        self.extend_field_names(fields, "files")
+        return fields
 
 
 class JobRunResponseSerializer(serializers.Serializer):
@@ -467,9 +677,9 @@ class JobClassSerializer(serializers.Serializer):
     )
     id = serializers.CharField(read_only=True, source="class_path")
     pk = serializers.SerializerMethodField(read_only=True)
-    name = serializers.CharField(max_length=255, read_only=True)
-    description = serializers.CharField(max_length=255, required=False, read_only=True)
-    test_methods = serializers.ListField(child=serializers.CharField(max_length=255))
+    name = serializers.CharField(max_length=CHARFIELD_MAX_LENGTH, read_only=True)
+    description = serializers.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False, read_only=True)
+    test_methods = serializers.ListField(child=serializers.CharField(max_length=CHARFIELD_MAX_LENGTH))
     vars = serializers.SerializerMethodField(read_only=True)
 
     @extend_schema_field(serializers.DictField)
@@ -502,22 +712,22 @@ class JobHookSerializer(NautobotModelSerializer):
         model = JobHook
         fields = "__all__"
 
-    def validate(self, data):
-        validated_data = super().validate(data)
+    def validate(self, attrs):
+        validated_attrs = super().validate(attrs)
 
         conflicts = JobHook.check_for_conflicts(
             instance=self.instance,
-            content_types=data.get("content_types"),
-            job=data.get("job"),
-            type_create=data.get("type_create"),
-            type_update=data.get("type_update"),
-            type_delete=data.get("type_delete"),
+            content_types=attrs.get("content_types"),
+            job=attrs.get("job"),
+            type_create=attrs.get("type_create"),
+            type_update=attrs.get("type_update"),
+            type_delete=attrs.get("type_delete"),
         )
 
         if conflicts:
             raise serializers.ValidationError(conflicts)
 
-        return validated_data
+        return validated_attrs
 
 
 class JobCreationSerializer(BaseModelSerializer):
@@ -529,22 +739,22 @@ class JobCreationSerializer(BaseModelSerializer):
     """
 
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:scheduledjob-detail")
-    name = serializers.CharField(max_length=255, required=False)
+    name = serializers.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False)
     start_time = serializers.DateTimeField(format=None, required=False)
 
     class Meta:
         model = ScheduledJob
         fields = ["url", "name", "start_time", "interval", "crontab"]
 
-    def validate(self, data):
-        data = super().validate(data)
+    def validate(self, attrs):
+        attrs = super().validate(attrs)
 
-        if data["interval"] in choices.JobExecutionType.SCHEDULE_CHOICES:
-            if "name" not in data:
+        if attrs["interval"] in choices.JobExecutionType.SCHEDULE_CHOICES:
+            if "name" not in attrs:
                 raise serializers.ValidationError({"name": "Please provide a name for the job schedule."})
 
-            if ("start_time" not in data and data["interval"] != choices.JobExecutionType.TYPE_CUSTOM) or (
-                "start_time" in data and data["start_time"] < models.ScheduledJob.earliest_possible_time()
+            if ("start_time" not in attrs and attrs["interval"] != choices.JobExecutionType.TYPE_CUSTOM) or (
+                "start_time" in attrs and attrs["start_time"] < models.ScheduledJob.earliest_possible_time()
             ):
                 raise serializers.ValidationError(
                     {
@@ -552,41 +762,46 @@ class JobCreationSerializer(BaseModelSerializer):
                     }
                 )
 
-            if data["interval"] == choices.JobExecutionType.TYPE_CUSTOM:
-                if data.get("crontab") is None:
+            if attrs["interval"] == choices.JobExecutionType.TYPE_CUSTOM:
+                if attrs.get("crontab") is None:
                     raise serializers.ValidationError({"crontab": "Please enter a valid crontab."})
                 try:
-                    models.ScheduledJob.get_crontab(data["crontab"])
+                    models.ScheduledJob.get_crontab(attrs["crontab"])
                 except Exception as e:
                     raise serializers.ValidationError({"crontab": e})
 
-        return data
+        return attrs
 
 
 class JobInputSerializer(serializers.Serializer):
     data = serializers.JSONField(required=False, default=dict)
     schedule = JobCreationSerializer(required=False)
     task_queue = serializers.CharField(required=False, allow_blank=True)
+    job_queue = serializers.CharField(required=False, allow_blank=True)
 
 
 class JobMultiPartInputSerializer(serializers.Serializer):
     """JobMultiPartInputSerializer is a "flattened" version of JobInputSerializer for use with multipart/form-data submissions which only accept key-value pairs"""
 
-    _schedule_name = serializers.CharField(max_length=255, required=False)
+    _schedule_name = serializers.CharField(max_length=CHARFIELD_MAX_LENGTH, required=False)
     _schedule_start_time = serializers.DateTimeField(format=None, required=False)
     _schedule_interval = ChoiceField(choices=JobExecutionType, required=False)
     _schedule_crontab = serializers.CharField(required=False, allow_blank=True)
     _task_queue = serializers.CharField(required=False, allow_blank=True)
+    _job_queue = serializers.CharField(required=False, allow_blank=True)
 
-    def validate(self, data):
-        data = super().validate(data)
+    def validate(self, attrs):
+        attrs = super().validate(attrs)
 
-        if "_schedule_interval" in data and data["_schedule_interval"] != JobExecutionType.TYPE_IMMEDIATELY:
-            if "_schedule_name" not in data:
+        if "_schedule_interval" in attrs and attrs["_schedule_interval"] != JobExecutionType.TYPE_IMMEDIATELY:
+            if "_schedule_name" not in attrs:
                 raise serializers.ValidationError({"_schedule_name": "Please provide a name for the job schedule."})
 
-            if ("_schedule_start_time" not in data and data["_schedule_interval"] != JobExecutionType.TYPE_CUSTOM) or (
-                "_schedule_start_time" in data and data["_schedule_start_time"] < ScheduledJob.earliest_possible_time()
+            if (
+                "_schedule_start_time" not in attrs and attrs["_schedule_interval"] != JobExecutionType.TYPE_CUSTOM
+            ) or (
+                "_schedule_start_time" in attrs
+                and attrs["_schedule_start_time"] < ScheduledJob.earliest_possible_time()
             ):
                 raise serializers.ValidationError(
                     {
@@ -594,15 +809,15 @@ class JobMultiPartInputSerializer(serializers.Serializer):
                     }
                 )
 
-            if data["_schedule_interval"] == JobExecutionType.TYPE_CUSTOM:
-                if data.get("_schedule_crontab") is None:
+            if attrs["_schedule_interval"] == JobExecutionType.TYPE_CUSTOM:
+                if attrs.get("_schedule_crontab") is None:
                     raise serializers.ValidationError({"_schedule_crontab": "Please enter a valid crontab."})
                 try:
-                    ScheduledJob.get_crontab(data["_schedule_crontab"])
+                    ScheduledJob.get_crontab(attrs["_schedule_crontab"])
                 except Exception as e:
                     raise serializers.ValidationError({"_schedule_crontab": e})
 
-        return data
+        return attrs
 
 
 class JobLogEntrySerializer(BaseModelSerializer):
@@ -624,6 +839,63 @@ class JobButtonSerializer(ValidatedModelSerializer, NotesSerializerMixin):
         fields = "__all__"
 
 
+#
+# Metadata
+#
+
+
+class MetadataTypeSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    content_types = ContentTypeField(
+        queryset=ContentType.objects.filter(FeatureQuery("metadata").get_query()),
+        many=True,
+    )
+
+    class Meta:
+        model = MetadataType
+        fields = "__all__"
+
+
+class MetadataChoiceSerializer(ValidatedModelSerializer):
+    class Meta:
+        model = MetadataChoice
+        fields = "__all__"
+
+
+class ObjectMetadataValueJSONField(serializers.JSONField):
+    """Special class to discern between itself and serializers.JSONField in NautobotCSVParser"""
+
+
+class ObjectMetadataSerializer(ValidatedModelSerializer):
+    assigned_object_type = ContentTypeField(
+        queryset=ContentType.objects.filter(FeatureQuery("metadata").get_query()),
+    )
+    assigned_object = serializers.SerializerMethodField()
+    value = ObjectMetadataValueJSONField(allow_null=True, required=False)
+
+    class Meta:
+        model = ObjectMetadata
+        fields = "__all__"
+
+    @extend_schema_field(
+        PolymorphicProxySerializer(
+            component_name="ObjectMetadataAssignedObject",
+            resource_type_field_name="object_type",
+            serializers=lambda: nested_serializers_for_models(FeatureQuery("metadata").list_subclasses()),
+            allow_null=True,
+        )
+    )
+    def get_assigned_object(self, obj):
+        if obj.assigned_object is None:
+            return None
+        try:
+            depth = get_nested_serializer_depth(self)
+            return return_nested_serializer_data_based_on_depth(
+                self, depth, obj, obj.assigned_object, "assigned_object"
+            )
+        except SerializerNotFound:
+            return None
+
+
 #
 # Notes
 #
@@ -636,7 +908,6 @@ class NoteSerializer(BaseModelSerializer):
     class Meta:
         model = Note
         fields = "__all__"
-        list_display_fields = ["note", "assigned_object_type", "assigned_object_id", "user"]
 
     @extend_schema_field(
         PolymorphicProxySerializer(
@@ -676,7 +947,6 @@ class ObjectChangeSerializer(BaseModelSerializer):
     class Meta:
         model = ObjectChange
         fields = "__all__"
-        list_display_fields = ["changed_object_id", "related_object_id", "related_object_type", "user"]
 
     @extend_schema_field(
         PolymorphicProxySerializer(
@@ -831,18 +1101,41 @@ class TagSerializer(NautobotModelSerializer):
             "color": {"help_text": "RGB color in hexadecimal (e.g. 00ff00)"},
         }
 
-    def validate(self, data):
-        data = super().validate(data)
+    def validate(self, attrs):
+        attrs = super().validate(attrs)
 
         # check if tag is assigned to any of the removed content_types
-        if self.instance is not None and self.instance.present_in_database and "content_types" in data:
-            content_types_id = [content_type.id for content_type in data["content_types"]]
+        if self.instance is not None and self.instance.present_in_database and "content_types" in attrs:
+            content_types_id = [content_type.id for content_type in attrs["content_types"]]
             errors = self.instance.validate_content_types_removal(content_types_id)
 
             if errors:
                 raise serializers.ValidationError(errors)
 
-        return data
+        return attrs
+
+
+#
+# Teams
+#
+
+
+class TeamSerializer(TaggedModelSerializerMixin, NautobotModelSerializer):
+    class Meta:
+        model = Team
+        fields = "__all__"
+        extra_kwargs = {
+            "contacts": {"required": False},
+            "email": {"default": ""},
+            "phone": {"default": ""},
+        }
+        # https://www.django-rest-framework.org/api-guide/validators/#optional-fields
+        validators = []
+
+    def validate(self, attrs):
+        validator = UniqueTogetherValidator(queryset=Team.objects.all(), fields=("name", "phone", "email"))
+        validator(attrs, self)
+        return super().validate(attrs)
 
 
 #
@@ -860,19 +1153,31 @@ class WebhookSerializer(ValidatedModelSerializer, NotesSerializerMixin):
         model = Webhook
         fields = "__all__"
 
-    def validate(self, data):
-        validated_data = super().validate(data)
+    def validate(self, attrs):
+        validated_attrs = super().validate(attrs)
 
         conflicts = Webhook.check_for_conflicts(
             instance=self.instance,
-            content_types=data.get("content_types"),
-            payload_url=data.get("payload_url"),
-            type_create=data.get("type_create"),
-            type_update=data.get("type_update"),
-            type_delete=data.get("type_delete"),
+            content_types=attrs.get("content_types"),
+            payload_url=attrs.get("payload_url"),
+            type_create=attrs.get("type_create"),
+            type_update=attrs.get("type_update"),
+            type_delete=attrs.get("type_delete"),
         )
 
         if conflicts:
             raise serializers.ValidationError(conflicts)
 
-        return validated_data
+        return validated_attrs
+
+
+#
+# More Git repositories
+#
+
+
+class GitRepositorySyncResponseSerializer(serializers.Serializer):
+    """Serializer representing responses from the GitRepository.sync() POST endpoint."""
+
+    message = serializers.CharField(read_only=True)
+    job_result = JobResultSerializer(read_only=True)
