@@ -1,16 +1,21 @@
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
+from django.utils.html import format_html
 from taggit.models import GenericUUIDTaggedItemBase
 
 from nautobot.core.choices import ColorChoices
+from nautobot.core.constants import CHARFIELD_MAX_LENGTH
 from nautobot.core.models import BaseManager, BaseModel
 from nautobot.core.models.fields import ColorField
 from nautobot.core.models.querysets import RestrictedQuerySet
-from nautobot.extras.models import ChangeLoggedModel, CustomFieldModel
-from nautobot.extras.models.mixins import NotesMixin
-from nautobot.extras.models.relationships import RelationshipModel
+from nautobot.extras.models.mixins import SavedViewMixin
 from nautobot.extras.utils import extras_features, TaggableClassesQuery
 
+# These imports are in this particular order because of circular import problems
+from .change_logging import ChangeLoggedModel
+from .customfields import CustomFieldModel
+from .mixins import ContactMixin, DynamicGroupsModelMixin, NotesMixin
+from .relationships import RelationshipModel
 
 #
 # Tags
@@ -31,8 +36,17 @@ class TagQuerySet(RestrictedQuerySet):
 @extras_features(
     "custom_validators",
 )
-class Tag(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel, NotesMixin):
-    name = models.CharField(max_length=100, unique=True)
+class Tag(
+    ChangeLoggedModel,
+    ContactMixin,
+    CustomFieldModel,
+    DynamicGroupsModelMixin,
+    NotesMixin,
+    RelationshipModel,
+    SavedViewMixin,
+    BaseModel,
+):
+    name = models.CharField(max_length=CHARFIELD_MAX_LENGTH, unique=True)
     content_types = models.ManyToManyField(
         to=ContentType,
         related_name="tags",
@@ -40,7 +54,7 @@ class Tag(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel, Not
     )
     color = ColorField(default=ColorChoices.COLOR_GREY)
     description = models.CharField(
-        max_length=200,
+        max_length=CHARFIELD_MAX_LENGTH,
         blank=True,
     )
 
@@ -52,6 +66,9 @@ class Tag(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel, Not
     class Meta:
         ordering = ["name"]
 
+    def get_color_display(self):
+        return format_html('<span class="label color-block" style="background-color: #{}">&nbsp;</span>', self.color)
+
     def validate_content_types_removal(self, content_types_id):
         """Validate content_types to be removed are not tagged to a model"""
         errors = {}
@@ -73,6 +90,7 @@ class TaggedItem(BaseModel, GenericUUIDTaggedItemBase):
     tag = models.ForeignKey(to=Tag, related_name="%(app_label)s_%(class)s_items", on_delete=models.CASCADE)
 
     documentation_static_path = "docs/user-guide/platform-functionality/tag.html"
+    is_metadata_associable_model = False
 
     natural_key_field_names = ["pk"]
 
