@@ -1,10 +1,9 @@
 from django.conf import settings
 
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import Job, BooleanVar, IntegerVar, StringVar, ObjectVar, get_task_logger
+from nautobot.extras.jobs import BooleanVar, get_task_logger, IntegerVar, Job, ObjectVar, StringVar
 from nautobot.extras.models import Role
 
-
 logger = get_task_logger(__name__)
 
 
@@ -19,7 +18,7 @@ class APITestJob(Job):
     var3 = BooleanVar()
     var4 = ObjectVar(model=Role)
 
-    def run(self, var1, var2, var3, var4):
+    def run(self, var1, var2, var3, var4):  # pylint:disable=arguments-differ
         logger.debug(var1)
         logger.info(var2)
         logger.warning(var3)
