@@ -1,8 +1,7 @@
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import DryRunVar, Job, get_task_logger
+from nautobot.extras.jobs import DryRunVar, get_task_logger, IntegerVar, Job
 from nautobot.extras.models import Status
 
-
 logger = get_task_logger(__name__)
 
 
@@ -12,8 +11,9 @@ class TestDryRun(Job):
     """
 
     dryrun = DryRunVar()
+    value = IntegerVar(required=False)
 
-    def run(self, dryrun):
+    def run(self, dryrun, value=None):  # pylint:disable=arguments-differ
         """
         Job function.
         """
