@@ -1,9 +1,8 @@
 from nautobot.core.celery import register_jobs
 from nautobot.dcim.models import Location, LocationType
-from nautobot.extras.jobs import JobHookReceiver, get_task_logger
+from nautobot.extras.jobs import get_task_logger, JobHookReceiver
 from nautobot.extras.models import Status
 
-
 logger = get_task_logger(__name__)
 
 
