@@ -1,8 +1,7 @@
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import Job, get_task_logger
+from nautobot.extras.jobs import get_task_logger, Job
 from nautobot.extras.models import Status
 
-
 logger = get_task_logger(__name__)
 
 
@@ -11,7 +10,7 @@ class TestModifyDB(Job):
     Job that modifies the database.
     """
 
-    def run(self):
+    def run(self):  # pylint: disable=arguments-differ
         """
         Job function.
         """
