@@ -1,7 +1,6 @@
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import Job, ObjectVar, get_task_logger
 from nautobot.dcim.models import Location
-
+from nautobot.extras.jobs import get_task_logger, Job, ObjectVar
 
 logger = get_task_logger(__name__)
 
@@ -13,7 +12,7 @@ class TestRequiredObjectVar(Job):
         required=True,
     )
 
-    def run(self, location):
+    def run(self, location):  # pylint: disable=arguments-differ
         logger.info("The Location that the user provided.", extra={"object": location})
         return "Nice Location!"
 
