@@ -2,8 +2,7 @@ import pstats
 import tempfile
 
 from nautobot.core.celery import register_jobs
-from nautobot.extras.jobs import Job, get_task_logger
-
+from nautobot.extras.jobs import get_task_logger, Job
 
 logger = get_task_logger(__name__)
 
@@ -15,7 +14,7 @@ class TestProfilingJob(Job):
 
     description = "Test profiling"
 
-    def run(self):
+    def run(self):  # pylint: disable=arguments-differ
         """
         Job function.
         """
