@@ -40,7 +40,7 @@ class ComputedFieldsTestCase(SeleniumTestCase):
         and checks it appears ONLY there!.
         """
         # Visit the device detail page
-        self.browser.visit(f'{self.live_server_url}{reverse("dcim:device", kwargs={"pk": self.device.pk})}')
+        self.browser.visit(f"{self.live_server_url}{reverse('dcim:device', kwargs={'pk': self.device.pk})}")
         # Check the computed field appears in the primary information tab
         self.assertTrue(self.browser.is_text_present("Device Computed Field"))
         self.assertTrue(self.browser.is_text_present(f"{self.device.name} is awesome!"))
@@ -52,7 +52,7 @@ class ComputedFieldsTestCase(SeleniumTestCase):
         self.computed_field.advanced_ui = True
         self.computed_field.save()
         # Visit the device detail page
-        self.browser.visit(f'{self.live_server_url}{reverse("dcim:device", kwargs={"pk": self.device.pk})}')
+        self.browser.visit(f"{self.live_server_url}{reverse('dcim:device', kwargs={'pk': self.device.pk})}")
         # Check the computed field does NOT appear in the primary information tab
         self.assertFalse(self.browser.is_text_present("Device Computed Field"))
         self.assertFalse(self.browser.is_text_present(f"{self.device.name} is awesome!"))
@@ -67,7 +67,7 @@ class ComputedFieldsTestCase(SeleniumTestCase):
         and then checks to see if it actually appears there.
         """
         self.browser.visit(f"{self.live_server_url}/dcim/devices/")
-        self.browser.find_by_xpath(".//button[@title='Configure table']").click()
+        self.browser.find_by_xpath(".//button[@data-original-title='Configure table']").click()
         select_option = self.browser.find_by_xpath(
             ".//select[@id='id_columns']/option[contains(text(), 'Device Computed Field')]"
         )
@@ -75,6 +75,6 @@ class ComputedFieldsTestCase(SeleniumTestCase):
         select_option.click()
         self.browser.find_by_xpath(".//input[@value='Save']").click()
         self.assertTrue(self.browser.is_text_present(f"{self.device.name} is awesome!"))
-        self.browser.find_by_xpath(".//button[@title='Configure table']").click()
+        self.browser.find_by_xpath(".//button[@data-original-title='Configure table']").click()
         self.browser.find_by_xpath(".//input[@value='Reset']").click()
         self.assertFalse(self.browser.is_text_present(f"{self.device.name} is awesome!"))
