@@ -4,7 +4,6 @@ from django.urls import reverse
 from django.utils.html import escape
 from rest_framework import status
 
-from example_plugin.signals import EXAMPLE_PLUGIN_CUSTOM_FIELD_DEFAULT, EXAMPLE_PLUGIN_CUSTOM_FIELD_NAME
 from nautobot.core.graphql import execute_query
 from nautobot.core.testing import APITestCase, TestCase
 from nautobot.core.testing.utils import post_data
@@ -13,10 +12,17 @@ from nautobot.core.utils.lookup import get_changes_for_model
 from nautobot.dcim.choices import InterfaceModeChoices
 from nautobot.dcim.models import Location, LocationType
 from nautobot.extras import context_managers
-from nautobot.extras.choices import CustomFieldTypeChoices, ObjectChangeActionChoices, ObjectChangeEventContextChoices
-from nautobot.extras.models import CustomField, CustomFieldChoice, ObjectChange, Status, Tag
+from nautobot.extras.choices import (
+    CustomFieldTypeChoices,
+    DynamicGroupTypeChoices,
+    ObjectChangeActionChoices,
+    ObjectChangeEventContextChoices,
+)
+from nautobot.extras.models import CustomField, CustomFieldChoice, DynamicGroup, ObjectChange, Status, Tag
 from nautobot.ipam.models import VLAN, VLANGroup
-from nautobot.virtualization.models import Cluster, ClusterType, VMInterface, VirtualMachine
+from nautobot.virtualization.models import Cluster, ClusterType, VirtualMachine, VMInterface
+
+from example_app.signals import EXAMPLE_APP_CUSTOM_FIELD_DEFAULT, EXAMPLE_APP_CUSTOM_FIELD_NAME
 
 
 class ChangeLogViewTest(ModelViewTestCase):
@@ -263,16 +269,15 @@ class ChangeLogAPITest(APITestCase):
             "custom_fields": {
                 "my_field": "ABC",
                 "my_field_select": "Bar",
-                EXAMPLE_PLUGIN_CUSTOM_FIELD_NAME: EXAMPLE_PLUGIN_CUSTOM_FIELD_DEFAULT,
+                EXAMPLE_APP_CUSTOM_FIELD_NAME: EXAMPLE_APP_CUSTOM_FIELD_DEFAULT,
             },
             "tags": [
                 {"name": self.tags[0].name},
                 {"name": self.tags[1].name},
             ],
         }
-        self.assertEqual(ObjectChange.objects.count(), 0)
         url = reverse("dcim-api:location-list")
-        self.add_permissions("dcim.add_location", "extras.view_status")
+        self.add_permissions("dcim.add_location", "dcim.view_locationtype", "extras.view_tag", "extras.view_status")
 
         response = self.client.post(url, data, format="json", **self.header)
         self.assertHttpStatus(response, status.HTTP_201_CREATED)
@@ -301,12 +306,11 @@ class ChangeLogAPITest(APITestCase):
             "custom_fields": {
                 "my_field": "DEF",
                 "my_field_select": "Foo",
-                EXAMPLE_PLUGIN_CUSTOM_FIELD_NAME: EXAMPLE_PLUGIN_CUSTOM_FIELD_DEFAULT,
+                EXAMPLE_APP_CUSTOM_FIELD_NAME: EXAMPLE_APP_CUSTOM_FIELD_DEFAULT,
             },
             "tags": [{"name": self.tags[2].name}],
         }
-        self.assertEqual(ObjectChange.objects.count(), 0)
-        self.add_permissions("dcim.change_location", "extras.view_status")
+        self.add_permissions("dcim.change_location", "extras.view_status", "dcim.view_locationtype", "extras.view_tag")
         url = reverse("dcim-api:location-detail", kwargs={"pk": location.pk})
 
         response = self.client.put(url, data, format="json", **self.header)
@@ -339,7 +343,6 @@ class ChangeLogAPITest(APITestCase):
             "description": "new description",
         }
 
-        self.assertEqual(ObjectChange.objects.count(), 0)
         self.add_permissions("dcim.change_location", "extras.view_status")
         url = reverse("dcim-api:location-detail", kwargs={"pk": location.pk})
 
@@ -367,7 +370,6 @@ class ChangeLogAPITest(APITestCase):
         )
         location.save()
         location.tags.set(self.tags[:2])
-        self.assertEqual(ObjectChange.objects.count(), 0)
         self.add_permissions("dcim.delete_location", "extras.view_status")
         url = reverse("dcim-api:location-detail", kwargs={"pk": location.pk})
         initial_count = Location.objects.count()
@@ -424,6 +426,7 @@ class ChangeLogAPITest(APITestCase):
         resp = execute_query(gql_payload, user=self.user).to_dict()
         self.assertFalse(resp["data"].get("error"))
         self.assertIsInstance(resp["data"].get("query"), list)
+        # ObjectChangeFactory creates records in the last year only; there shouldn't be any in this filtered response.
         self.assertEqual(len(resp["data"].get("query")), 0)
 
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
@@ -454,8 +457,7 @@ class ChangeLogAPITest(APITestCase):
             "status": self.statuses[0].pk,
             "location_type": location_type.pk,
         }
-        self.assertEqual(ObjectChange.objects.count(), 0)
-        self.add_permissions("dcim.add_location")
+        self.add_permissions("dcim.add_location", "dcim.view_locationtype", "extras.view_status")
         url = reverse("dcim-api:location-list")
 
         response = self.client.post(url, location_payload, format="json", **self.header)
@@ -490,15 +492,14 @@ class ChangeLogAPITest(APITestCase):
         )
 
         payload = {"tagged_vlans": [str(tagged_vlan.pk)], "description": "test vm interface m2m change"}
-        self.assertEqual(ObjectChange.objects.count(), 0)
-        self.add_permissions("virtualization.change_vminterface", "ipam.change_vlan")
+        self.add_permissions("virtualization.change_vminterface", "ipam.change_vlan", "ipam.view_vlan")
         url = reverse("virtualization-api:vminterface-detail", kwargs={"pk": vm_interface.pk})
         response = self.client.patch(url, payload, format="json", **self.header)
         vm_interface.refresh_from_db()
         self.assertHttpStatus(response, status.HTTP_200_OK)
 
         oc = get_changes_for_model(vm_interface).first()
-        self.assertEqual(ObjectChange.objects.count(), 1)
+        self.assertEqual(get_changes_for_model(vm_interface).count(), 1)
         self.assertEqual(oc.user_id, self.user.pk)
         self.assertEqual(vm_interface.description, "test vm interface m2m change")
         self.assertSequenceEqual(list(vm_interface.tagged_vlans.all()), [tagged_vlan])
@@ -509,6 +510,17 @@ class ObjectChangeModelTest(TestCase):  # TODO: change to BaseModelTestCase once
     def setUpTestData(cls):
         cls.location_status = Status.objects.get_for_model(Location).first()
 
+    def test_opt_out(self):
+        """Hidden static group associations can "opt out" of change logging."""
+        dg = DynamicGroup.objects.exclude(group_type=DynamicGroupTypeChoices.TYPE_STATIC).first()
+        # Force reassignment of all cached memberships:
+        members = list(dg.members)
+        dg._set_members([])
+        dg._set_members(members)
+
+        for sga in dg.static_group_associations(manager="all_objects").all():
+            self.assertIsNone(get_changes_for_model(sga).first())
+
     def test_get_snapshots(self):
         with context_managers.web_request_context(self.user):
             location_type = LocationType.objects.get(name="Campus")
