@@ -1,11 +1,13 @@
 import re
 
-import netaddr
 from django.core.exceptions import ValidationError
+from django.core.validators import validate_ipv46_address
 from django.db.models import ProtectedError, Q
+import netaddr
 
 from nautobot.core.models.querysets import RestrictedQuerySet
 from nautobot.core.utils.data import merge_dicts_without_collision
+from nautobot.ipam.mixins import LocationToLocationsQuerySetMixin
 
 
 class RIRQuerySet(RestrictedQuerySet):
@@ -55,7 +57,7 @@ class BaseNetworkQuerySet(RestrictedQuerySet):
             return call_map[version](search)
 
         except netaddr.core.AddrFormatError:
-            ver_map = {4: "0/32", 6: "::/128"}
+            ver_map = {4: "0.0.0.0/32", 6: "::/128"}
             return netaddr.IPNetwork(ver_map[version])
 
     def _check_and_prep_ipv6(self, search):
@@ -187,50 +189,50 @@ class BaseNetworkQuerySet(RestrictedQuerySet):
 
         return self.filter(the_filter)
 
+    def _build_prefix_and_last_ip(self, ip):
+        if isinstance(ip, str):
+            ip = netaddr.IPNetwork(ip)
+        last_ip = self._get_last_ip(ip)
+        return ip, last_ip
 
-class PrefixQuerySet(BaseNetworkQuerySet):
+
+class PrefixQuerySet(LocationToLocationsQuerySetMixin, BaseNetworkQuerySet):
     """Queryset for `Prefix` objects."""
 
-    def net_equals(self, prefix):
-        prefix = netaddr.IPNetwork(prefix)
-        last_ip = self._get_last_ip(prefix)
-        return self.filter(prefix_length=prefix.prefixlen, network=prefix.network, broadcast=last_ip)
+    def net_equals(self, *prefixes):
+        query = Q()
+        for prefix in prefixes:
+            prefix, last_ip = self._build_prefix_and_last_ip(prefix)
+            query |= Q(prefix_length=prefix.prefixlen, network=prefix.network, broadcast=last_ip)
+        return self.filter(query) if query else self.none()
 
-    def net_contained(self, prefix):
-        prefix = netaddr.IPNetwork(prefix)
-        last_ip = self._get_last_ip(prefix)
-        return self.filter(
-            prefix_length__gt=prefix.prefixlen,
-            network__gte=prefix.network,
-            broadcast__lte=last_ip,
-        )
+    def net_contained(self, *prefixes):
+        query = Q()
+        for prefix in prefixes:
+            prefix, last_ip = self._build_prefix_and_last_ip(prefix)
+            query |= Q(prefix_length__gt=prefix.prefixlen, network__gte=prefix.network, broadcast__lte=last_ip)
+        return self.filter(query) if query else self.none()
 
-    def net_contained_or_equal(self, prefix):
-        prefix = netaddr.IPNetwork(prefix)
-        last_ip = self._get_last_ip(prefix)
-        return self.filter(
-            prefix_length__gte=prefix.prefixlen,
-            network__gte=prefix.network,
-            broadcast__lte=last_ip,
-        )
+    def net_contained_or_equal(self, *prefixes):
+        query = Q()
+        for prefix in prefixes:
+            prefix, last_ip = self._build_prefix_and_last_ip(prefix)
+            query |= Q(prefix_length__gte=prefix.prefixlen, network__gte=prefix.network, broadcast__lte=last_ip)
+        return self.filter(query) if query else self.none()
 
-    def net_contains(self, prefix):
-        prefix = netaddr.IPNetwork(prefix)
-        last_ip = self._get_last_ip(prefix)
-        return self.filter(
-            prefix_length__lt=prefix.prefixlen,
-            network__lte=prefix.network,
-            broadcast__gte=last_ip,
-        )
+    def net_contains(self, *prefixes):
+        query = Q()
+        for prefix in prefixes:
+            prefix, last_ip = self._build_prefix_and_last_ip(prefix)
+            query |= Q(prefix_length__lt=prefix.prefixlen, network__lte=prefix.network, broadcast__gte=last_ip)
+        return self.filter(query) if query else self.none()
 
-    def net_contains_or_equals(self, prefix):
-        prefix = netaddr.IPNetwork(prefix)
-        last_ip = self._get_last_ip(prefix)
-        return self.filter(
-            prefix_length__lte=prefix.prefixlen,
-            network__lte=prefix.network,
-            broadcast__gte=last_ip,
-        )
+    def net_contains_or_equals(self, *prefixes):
+        query = Q()
+        for prefix in prefixes:
+            prefix, last_ip = self._build_prefix_and_last_ip(prefix)
+            query |= Q(prefix_length__lte=prefix.prefixlen, network__lte=prefix.network, broadcast__gte=last_ip)
+        return self.filter(query) if query else self.none()
 
     def get(self, *args, **kwargs):
         """
@@ -305,7 +307,7 @@ class PrefixQuerySet(BaseNetworkQuerySet):
             return super().delete(*args, **kwargs)
         except ProtectedError as err:
             # This will be either IPAddress or Prefix.
-            protected_instance = tuple(err.protected_objects)[0]
+            protected_instance = next(iter(err.protected_objects))
             protected_model = protected_instance._meta.model
             protected_parent = protected_instance.parent
             new_parent = protected_parent.parent_id
@@ -396,6 +398,37 @@ class IPAddressQuerySet(BaseNetworkQuerySet):
         """
         return super().order_by("host")
 
+    def get_or_create(self, defaults=None, **kwargs):
+        from nautobot.ipam.models import get_default_namespace, Prefix
+
+        parent = kwargs.get("parent")
+        namespace = kwargs.pop("namespace", None)
+        host = kwargs.get("host")
+        mask_length = kwargs.get("mask_length")
+        address = kwargs.get("address")
+        if host is None and address is not None:
+            address = netaddr.IPNetwork(address)
+            host = str(address.ip)
+            mask_length = address.prefixlen
+
+        # If `host` or `mask_length` is None skip; then there is no way of getting the closest parent;
+        if parent is None and host is not None and mask_length is not None:
+            if namespace is None:
+                namespace = get_default_namespace()
+            cidr = f"{host}/{mask_length}"
+
+            try:
+                validate_ipv46_address(host)
+            except ValidationError as err:
+                raise ValidationError({"host": err.error_list}) from err
+            try:
+                netaddr.IPNetwork(cidr)
+            except netaddr.AddrFormatError as err:
+                raise ValidationError(f"{cidr} does not appear to be an IPv4 or IPv6 network.") from err
+            parent = Prefix.objects.filter(namespace=namespace).get_closest_parent(cidr=cidr, include_self=True)
+            kwargs["parent"] = parent
+        return super().get_or_create(defaults=defaults, **kwargs)
+
     def string_search(self, search):
         """
         Interpret a search string and return useful results.
@@ -422,15 +455,14 @@ class IPAddressQuerySet(BaseNetworkQuerySet):
 
         return self.filter(the_filter)
 
-    def net_host_contained(self, network):
+    def net_host_contained(self, *networks):
         # consider only host ip address when
         # filtering for membership in |network|
-        network = netaddr.IPNetwork(network)
-        last_ip = self._get_last_ip(network)
-        return self.filter(
-            host__lte=last_ip,
-            host__gte=network.network,
-        )
+        query = Q()
+        for ip in networks:
+            ip, last_ip = self._build_prefix_and_last_ip(ip)
+            query |= Q(host__lte=last_ip, host__gte=ip.network)
+        return self.filter(query) if query else self.none()
 
     def net_in(self, networks):
         # for a tuple of IP addresses, filter queryset for matches.
@@ -475,3 +507,7 @@ class IPAddressQuerySet(BaseNetworkQuerySet):
             q |= Q(pk__in=pk_values)
 
         return super().filter(q)
+
+
+class VLANQuerySet(LocationToLocationsQuerySetMixin, RestrictedQuerySet):
+    """Queryset for `VLAN` objects."""
