@@ -1,5 +1,6 @@
-import django_tables2 as tables
+from django.db.models import Prefetch
 from django.utils.safestring import mark_safe
+import django_tables2 as tables
 from django_tables2.utils import Accessor
 
 from nautobot.core.tables import (
@@ -18,6 +19,8 @@ from nautobot.dcim.utils import cable_status_color_css
 from nautobot.extras.tables import RoleTableMixin, StatusTableMixin
 from nautobot.tenancy.tables import TenantColumn
 from nautobot.virtualization.models import VMInterface
+from nautobot.virtualization.tables import VMInterfaceTable
+
 from .models import (
     IPAddress,
     IPAddressToInterface,
@@ -33,35 +36,21 @@ from .models import (
     VRFPrefixAssignment,
 )
 
-AVAILABLE_LABEL = mark_safe('<span class="label label-success">Available</span>')  # noqa: S308
+AVAILABLE_LABEL = mark_safe('<span class="label label-success">Available</span>')
 
 UTILIZATION_GRAPH = """
 {% load helpers %}
 {% if record.present_in_database %}{% utilization_graph record.get_utilization %}{% else %}&mdash;{% endif %}
 """
 
-PREFIX_LINK = """
-{% load helpers %}
-{% for i in record.ancestors.count|as_range %}
-    <i class="mdi mdi-circle-small"></i>
-{% endfor %}
-    <a href="\
-{% if record.present_in_database %}\
-{% url 'ipam:prefix' pk=record.pk %}\
-{% else %}\
-{% url 'ipam:prefix_add' %}\
-?prefix={{ record }}&namespace={{ object.namespace.pk }}\
-{% if object.location %}&location={{ object.location.pk }}{% endif %}\
-{% if object.tenant %}&tenant_group={{ object.tenant.tenant_group.pk }}&tenant={{ object.tenant.pk }}{% endif %}\
-{% endif %}\
-">{{ record.prefix }}</a>
-"""
 
+# record: the Prefix being rendered in this row
+# object: the base ancestor Prefix, in the case of PrefixDetailTable, else None
 PREFIX_COPY_LINK = """
 {% load helpers %}
-{% for i in record.ancestors.count|as_range %}
-    <i class="mdi mdi-circle-small"></i>
-{% endfor %}
+{% if not table.hide_hierarchy_ui %}
+{% tree_hierarchy_ui_representation record.ancestors.count|as_range table.hide_hierarchy_ui base_tree_depth|default:0 %}
+{% endif %}
 <span class="hover_copy">
   <a href="\
 {% if record.present_in_database %}\
@@ -69,7 +58,7 @@ PREFIX_COPY_LINK = """
 {% else %}\
 {% url 'ipam:prefix_add' %}\
 ?prefix={{ record }}&namespace={{ object.namespace.pk }}\
-{% if object.location %}&location={{ object.location.pk }}{% endif %}\
+{% for loc in object.locations.all %}&locations={{ loc.pk }}{% endfor %}\
 {% if object.tenant %}&tenant_group={{ object.tenant.tenant_group.pk }}&tenant={{ object.tenant.pk }}{% endif %}\
 {% endif %}\
 " id="copy_{{record.id}}">{{ record.prefix }}</a>
@@ -93,7 +82,7 @@ IPADDRESS_LINK = """
 {% elif perms.ipam.add_ipaddress %}
     <a href="\
 {% url 'ipam:ipaddress_add' %}\
-?address={{ record.1 }}\
+?address={{ record.1 }}&namespace={{ object.namespace.pk }}\
 {% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}\
 {% if object.tenant %}&tenant={{ object.tenant.pk }}{% endif %}\
 " class="btn btn-xs btn-success">\
@@ -115,7 +104,7 @@ IPADDRESS_COPY_LINK = """
 {% elif perms.ipam.add_ipaddress %}
     <a href="\
 {% url 'ipam:ipaddress_add' %}\
-?address={{ record.1 }}\
+?address={{ record.1 }}&namespace={{ object.namespace.pk }}\
 {% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}\
 {% if object.tenant %}&tenant={{ object.tenant.pk }}{% endif %}\
 " class="btn btn-xs btn-success">\
@@ -177,10 +166,9 @@ VLAN_LINK = """
 {% url 'ipam:vlan_add' %}\
 ?vid={{ record.vid }}&vlan_group={{ vlan_group.pk }}\
 {% if vlan_group.location %}&location={{ vlan_group.location.pk }}{% endif %}\
-{% if vlan_group.location %}&location={{ vlan_group.location.pk }}{% endif %}\
-" class="btn btn-xs btn-success">{{ record.available }} VLAN{{ record.available|pluralize }} available</a>\
+" class="btn btn-xs btn-success">{{ record.available }} VLAN{{ record.available|pluralize }} available ({{ record.range }})</a>\
 {% else %}
-    {{ record.available }} VLAN{{ record.available|pluralize }} available
+    {{ record.available }} VLAN{{ record.available|pluralize }} available ({{ record.range }})
 {% endif %}
 """
 
@@ -192,14 +180,6 @@ VLAN_PREFIXES = """
 {% endfor %}
 """
 
-VLAN_ROLE_LINK = """
-{% if record.role %}
-    <a href="{% url 'ipam:vlan_list' %}?role={{ record.role.name }}">{{ record.role }}</a>
-{% else %}
-    &mdash;
-{% endif %}
-"""
-
 VLANGROUP_ADD_VLAN = """
 {% with next_vid=record.get_next_available_vid %}
     {% if next_vid and perms.ipam.add_vlan %}
@@ -234,10 +214,10 @@ class NamespaceTable(BaseTable):
 #
 
 
-class VRFTable(BaseTable):
+class VRFTable(StatusTableMixin, BaseTable):
     pk = ToggleColumn()
     name = tables.LinkColumn()
-    # rd = tables.Column(verbose_name="RD")
+    rd = tables.Column(verbose_name="RD")
     tenant = TenantColumn()
     import_targets = tables.TemplateColumn(template_code=VRF_TARGETS, orderable=False)
     export_targets = tables.TemplateColumn(template_code=VRF_TARGETS, orderable=False)
@@ -248,7 +228,8 @@ class VRFTable(BaseTable):
         fields = (
             "pk",
             "name",
-            # "rd",
+            "rd",
+            "status",
             "namespace",
             "tenant",
             "description",
@@ -256,8 +237,7 @@ class VRFTable(BaseTable):
             "export_targets",
             "tags",
         )
-        # default_columns = ("pk", "name", "rd", "namespace", "tenant", "description")
-        default_columns = ("pk", "name", "namespace", "tenant", "description")
+        default_columns = ("pk", "name", "rd", "status", "namespace", "tenant", "description")
 
 
 class VRFDeviceAssignmentTable(BaseTable):
@@ -269,13 +249,27 @@ class VRFDeviceAssignmentTable(BaseTable):
         linkify=lambda record: record.vrf.namespace.get_absolute_url(),
         accessor="vrf.namespace.name",
     )
-    device = tables.Column(
-        linkify=lambda record: record.device.get_absolute_url(), accessor="device.name", verbose_name="Device"
+    related_object_type = tables.TemplateColumn(
+        template_code="""
+        {% if record.device %}
+            Device
+        {% elif record.virtual_machine %}
+            Virtual Machine
+        {% else %}
+            Virtual Device Context
+        {% endif %}
+        """
     )
-    virtual_machine = tables.Column(
-        linkify=lambda record: record.virtual_machine.get_absolute_url(),
-        accessor="virtual_machine.name",
-        verbose_name="Virtual Machine",
+    related_object_name = tables.TemplateColumn(
+        template_code="""
+        {% if record.device %}
+            <a href="{{ record.device.get_absolute_url }}">{{ record.device.name }}</a>
+        {% elif record.virtual_machine %}
+            <a href="{{ record.virtual_machine.get_absolute_url }}">{{ record.virtual_machine.name }}</a>
+        {% else %}
+            <a href="{{ record.virtual_device_context.get_absolute_url }}">{{ record.virtual_device_context.name }}</a>
+        {% endif %}
+        """
     )
     rd = tables.Column(verbose_name="VRF RD")
     tenant = TenantColumn(accessor="vrf.tenant")
@@ -283,7 +277,7 @@ class VRFDeviceAssignmentTable(BaseTable):
     class Meta(BaseTable.Meta):
         model = VRFDeviceAssignment
         orderable = False
-        fields = ("vrf", "namespace", "device", "virtual_machine", "rd", "tenant")
+        fields = ("vrf", "related_object_type", "related_object_name", "namespace", "rd", "tenant")
 
 
 class VRFPrefixAssignmentTable(BaseTable):
@@ -362,46 +356,60 @@ class PrefixTable(StatusTableMixin, RoleTableMixin, BaseTable):
     prefix = tables.TemplateColumn(
         template_code=PREFIX_COPY_LINK, attrs={"td": {"class": "text-nowrap"}}, order_by=("network", "prefix_length")
     )
-    # vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
+    vrf_count = LinkedCountColumn(
+        viewname="ipam:vrf_list",
+        url_params={"prefix": "pk"},
+        display_field="name",
+        reverse_lookup="prefixes",
+        verbose_name="VRFs",
+    )
     tenant = TenantColumn()
-    location = tables.Column(linkify=True)
     namespace = tables.Column(linkify=True)
     vlan = tables.Column(linkify=True, verbose_name="VLAN")
     rir = tables.Column(linkify=True)
-    children = tables.Column(accessor="descendants_count")
+    children = tables.Column(accessor="descendants_count", orderable=False)
     date_allocated = tables.DateTimeColumn()
+    location_count = LinkedCountColumn(
+        viewname="dcim:location_list", url_params={"prefixes": "pk"}, display_field="name", verbose_name="Locations"
+    )
+    cloud_networks_count = LinkedCountColumn(
+        viewname="cloud:cloudnetwork_list", url_params={"prefixes": "pk"}, verbose_name="Cloud Networks"
+    )
+    actions = ButtonsColumn(Prefix)
 
     class Meta(BaseTable.Meta):
         model = Prefix
-        orderable = False
         fields = (
             "pk",
             "prefix",
             "type",
             "status",
             "children",
-            # "vrf",
+            "vrf_count",
             "namespace",
             "tenant",
-            "location",
+            "location_count",
+            "cloud_networks_count",
             "vlan",
             "role",
             "rir",
             "date_allocated",
             "description",
+            "actions",
         )
         default_columns = (
             "pk",
             "prefix",
             "type",
             "status",
-            # "vrf",
+            "vrf_count",
             "namespace",
             "tenant",
-            "location",
+            "location_count",
             "vlan",
             "role",
             "description",
+            "actions",
         )
         row_attrs = {
             "class": lambda record: "success" if not record.present_in_database else "",
@@ -413,6 +421,15 @@ class PrefixDetailTable(PrefixTable):
     tenant = TenantColumn()
     tags = TagColumn(url_name="ipam:prefix_list")
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.add_conditional_prefetch(
+            "utilization",
+            prefetch=Prefetch(
+                "children", queryset=Prefix.objects.only("network", "prefix_length", "parent_id").order_by()
+            ),
+        )
+
     class Meta(PrefixTable.Meta):
         fields = (
             "pk",
@@ -421,10 +438,10 @@ class PrefixDetailTable(PrefixTable):
             "type",
             "status",
             "children",
-            # "vrf",
+            "vrf_count",
             "utilization",
             "tenant",
-            "location",
+            "location_count",
             "vlan",
             "role",
             "description",
@@ -437,10 +454,10 @@ class PrefixDetailTable(PrefixTable):
             "type",
             "status",
             "children",
-            # "vrf",
+            "vrf_count",
             "utilization",
             "tenant",
-            "location",
+            "location_count",
             "vlan",
             "role",
             "description",
@@ -459,12 +476,27 @@ class IPAddressTable(StatusTableMixin, RoleTableMixin, BaseTable):
     )
     tenant = TenantColumn()
     parent__namespace = tables.Column(linkify=True)
-    interface_count = tables.Column(verbose_name="Interfaces")
-    interface_parent_count = tables.Column(verbose_name="Devices")
+    interface_count = LinkedCountColumn(
+        viewname="dcim:interface_list", url_params={"ip_addresses": "pk"}, verbose_name="Interfaces"
+    )
+    # TODO: what about interfaces assigned to modules?
+    interface_parent_count = LinkedCountColumn(
+        viewname="dcim:device_list",
+        url_params={"ip_addresses": "pk"},
+        reverse_lookup="interfaces__ip_addresses",
+        distinct=True,
+        verbose_name="Devices",
+    )
     vm_interface_count = LinkedCountColumn(
         viewname="virtualization:vminterface_list", url_params={"ip_addresses": "pk"}, verbose_name="VM Interfaces"
     )
-    vm_interface_parent_count = tables.Column(verbose_name="Virtual Machines")
+    vm_interface_parent_count = LinkedCountColumn(
+        viewname="virtualization:virtualmachine_list",
+        url_params={"ip_addresses": "pk"},
+        reverse_lookup="interfaces__ip_addresses",
+        distinct=True,
+        verbose_name="Virtual Machines",
+    )
 
     class Meta(BaseTable.Meta):
         model = IPAddress
@@ -489,7 +521,7 @@ class IPAddressTable(StatusTableMixin, RoleTableMixin, BaseTable):
 
 
 class IPAddressDetailTable(IPAddressTable):
-    nat_inside = tables.Column(linkify=True, orderable=False, verbose_name="NAT (Inside)")
+    nat_inside = tables.Column(linkify=True, verbose_name="NAT (Inside)")
     tenant = TenantColumn()
     tags = TagColumn(url_name="ipam:ipaddress_list")
     assigned = BooleanColumn(accessor="assigned_count")
@@ -622,6 +654,13 @@ class IPAddressInterfaceTable(InterfaceTable):
         }
 
 
+class IPAddressVMInterfaceTable(VMInterfaceTable):
+    class Meta(VMInterfaceTable.Meta):
+        row_attrs = {
+            "data-name": lambda record: record.name,
+        }
+
+
 #
 # IPAddress to Interface
 #
@@ -668,8 +707,8 @@ class VLANGroupTable(BaseTable):
 
     class Meta(BaseTable.Meta):
         model = VLANGroup
-        fields = ("pk", "name", "location", "vlan_count", "description", "actions")
-        default_columns = ("pk", "name", "location", "vlan_count", "description", "actions")
+        fields = ("pk", "name", "location", "range", "vlan_count", "description", "actions")
+        default_columns = ("pk", "name", "range", "location", "vlan_count", "description", "actions")
 
 
 #
@@ -680,8 +719,13 @@ class VLANGroupTable(BaseTable):
 class VLANTable(StatusTableMixin, RoleTableMixin, BaseTable):
     pk = ToggleColumn()
     vid = tables.TemplateColumn(template_code=VLAN_LINK, verbose_name="ID")
-    location = tables.Column(linkify=True)
     vlan_group = tables.Column(linkify=True)
+    location_count = LinkedCountColumn(
+        viewname="dcim:location_list",
+        url_params={"vlans": "pk"},
+        display_field="name",
+        verbose_name="Locations",
+    )
     tenant = TenantColumn()
 
     class Meta(BaseTable.Meta):
@@ -689,9 +733,9 @@ class VLANTable(StatusTableMixin, RoleTableMixin, BaseTable):
         fields = (
             "pk",
             "vid",
-            "location",
             "vlan_group",
             "name",
+            "location_count",
             "tenant",
             "status",
             "role",
@@ -711,10 +755,10 @@ class VLANDetailTable(VLANTable):
         fields = (
             "pk",
             "vid",
-            "location",
             "vlan_group",
             "name",
             "prefixes",
+            "location_count",
             "tenant",
             "status",
             "role",
@@ -724,10 +768,10 @@ class VLANDetailTable(VLANTable):
         default_columns = (
             "pk",
             "vid",
-            "location",
             "vlan_group",
             "name",
             "prefixes",
+            "location_count",
             "tenant",
             "status",
             "role",
@@ -765,25 +809,28 @@ class VLANVirtualMachinesTable(VLANMembersTable):
         fields = ("virtual_machine", "name", "tagged", "actions")
 
 
-class InterfaceVLANTable(StatusTableMixin, BaseTable):
+class InterfaceVLANTable(StatusTableMixin, RoleTableMixin, BaseTable):
     """
     List VLANs assigned to a specific Interface.
     """
 
     vid = tables.LinkColumn(viewname="ipam:vlan", args=[Accessor("pk")], verbose_name="ID")
     tagged = BooleanColumn()
-    location = tables.Column(linkify=True)
     vlan_group = tables.Column(accessor=Accessor("vlan_group__name"), verbose_name="Group")
     tenant = TenantColumn()
-    role = tables.TemplateColumn(template_code=VLAN_ROLE_LINK)
+    location_count = LinkedCountColumn(
+        viewname="dcim:location_list",
+        url_params={"vlans": "pk"},
+        verbose_name="Locations",
+    )
 
     class Meta(BaseTable.Meta):
         model = VLAN
         fields = (
             "vid",
             "tagged",
-            "location",
             "vlan_group",
+            "location_count",
             "name",
             "tenant",
             "status",
