@@ -1,7 +1,7 @@
 from django.contrib.contenttypes.models import ContentType
 from django.db.models import Q
 
-from nautobot.core.testing import TestCase, FilterTestCases
+from nautobot.core.testing import FilterTestCases, TestCase
 from nautobot.dcim.choices import InterfaceTypeChoices
 from nautobot.dcim.models import (
     Device,
@@ -10,6 +10,7 @@ from nautobot.dcim.models import (
     Location,
     LocationType,
     Manufacturer,
+    VirtualDeviceContext,
 )
 from nautobot.extras.models import Role, Status, Tag
 from nautobot.ipam.choices import PrefixTypeChoices, ServiceProtocolChoices
@@ -17,24 +18,32 @@ from nautobot.ipam.filters import (
     IPAddressFilterSet,
     IPAddressToInterfaceFilterSet,
     PrefixFilterSet,
+    PrefixLocationAssignmentFilterSet,
     RIRFilterSet,
     RouteTargetFilterSet,
     ServiceFilterSet,
     VLANFilterSet,
     VLANGroupFilterSet,
+    VLANLocationAssignmentFilterSet,
+    VRFDeviceAssignmentFilterSet,
     VRFFilterSet,
+    VRFPrefixAssignmentFilterSet,
 )
 from nautobot.ipam.models import (
     IPAddress,
     IPAddressToInterface,
+    Namespace,
     Prefix,
+    PrefixLocationAssignment,
     RIR,
     RouteTarget,
     Service,
     VLAN,
     VLANGroup,
+    VLANLocationAssignment,
     VRF,
-    Namespace,
+    VRFDeviceAssignment,
+    VRFPrefixAssignment,
 )
 from nautobot.tenancy.models import Tenant
 from nautobot.virtualization.models import (
@@ -59,88 +68,87 @@ class VRFTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilterT
     queryset = VRF.objects.all()
     filterset = VRFFilterSet
     tenancy_related_name = "vrfs"
+    # skip testing "rd" attribute for generic q filter test as it's not trivially modifiable
+    exclude_q_filter_predicates = ["rd"]
+    generic_filter_tests = (
+        # ("device", "devices__id"),
+        # ("device", "devices__name"),
+        ("export_targets", "export_targets__id"),
+        ("export_targets", "export_targets__name"),
+        ("import_targets", "import_targets__id"),
+        ("import_targets", "import_targets__name"),
+        ("prefix", "prefixes__id"),
+        ("name",),
+        ("namespace", "namespace__id"),
+        ("namespace", "namespace__name"),
+        ("rd",),
+        # ("virtual_machines", "virtual_machines__id"),
+        # ("virtual_machines", "virtual_machines__name"),
+    )
 
     @classmethod
     def setUpTestData(cls):
         instance = cls.queryset.first()
         instance.tags.set(Tag.objects.all()[:2])
 
-    def test_name(self):
-        names = list(self.queryset.values_list("name", flat=True))[:2]
-        params = {"name": names}
-        self.assertQuerysetEqual(self.filterset(params, self.queryset).qs, self.queryset.filter(name__in=names))
-
-    def test_rd(self):
-        vrfs = self.queryset.filter(rd__isnull=False)[:2]
-        params = {"rd": [vrfs[0].rd, vrfs[1].rd]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_import_targets(self):
-        route_targets = list(RouteTarget.objects.filter(importing_vrfs__isnull=False).distinct())[:2]
-        params = {"import_targets": [route_targets[0].pk, route_targets[1].name]}
+    def test_prefix_filter_by_string(self):
+        """Test filtering by prefix strings as an alternative to pk."""
+        prefix = self.queryset.filter(prefixes__isnull=False).first().prefixes.first()
+        params = {"prefix": [prefix.prefix]}
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs,
-            self.queryset.filter(import_targets__in=route_targets).distinct(),
+            self.queryset.filter(prefixes__network=prefix.network, prefixes__prefix_length=prefix.prefix_length),
             ordered=False,
         )
 
-    def test_export_targets(self):
-        route_targets = list(RouteTarget.objects.filter(exporting_vrfs__isnull=False).distinct())[:2]
-        params = {"export_targets": [route_targets[0].pk, route_targets[1].name]}
+
+class VRFPrefixAssignmentTestCase(FilterTestCases.FilterTestCase):
+    queryset = VRFPrefixAssignment.objects.all()
+    filterset = VRFPrefixAssignmentFilterSet
+    generic_filter_tests = (
+        ("prefix", "prefix__id"),
+        ("vrf", "vrf__id"),
+        ("vrf", "vrf__name"),
+    )
+
+    def test_prefix_filter_by_string(self):
+        """Test filtering by prefix strings as an alternative to pk."""
+        prefix = self.queryset.first().prefix
+        params = {"prefix": [prefix.prefix]}
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs,
-            self.queryset.filter(export_targets__in=route_targets).distinct(),
+            self.queryset.filter(prefix__network=prefix.network, prefix__prefix_length=prefix.prefix_length),
             ordered=False,
         )
 
-    def test_search(self):
-        value = self.queryset.values_list("pk", flat=True)[0]
-        params = {"q": value}
-        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
-
 
 class RouteTargetTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilterTestCaseMixin):
     queryset = RouteTarget.objects.all()
     filterset = RouteTargetFilterSet
     tenancy_related_name = "route_targets"
+    # skip testing "name" attribute for generic q filter test as it's not trivially modifiable
+    exclude_q_filter_predicates = ["name"]
+    generic_filter_tests = (
+        ("exporting_vrfs", "exporting_vrfs__id"),
+        ("exporting_vrfs", "exporting_vrfs__rd"),
+        ("importing_vrfs", "importing_vrfs__id"),
+        ("importing_vrfs", "importing_vrfs__rd"),
+        ("name",),
+    )
 
     @classmethod
     def setUpTestData(cls):
         instance = cls.queryset.first()
         instance.tags.set(Tag.objects.all()[:2])
 
-    def test_name(self):
-        params = {"name": [self.queryset[0].name, self.queryset[1].name, self.queryset[2].name]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
 
-    def test_importing_vrfs(self):
-        vrfs = list(VRF.objects.filter(import_targets__isnull=False, rd__isnull=False).distinct())[:2]
-        params = {"importing_vrfs": [vrfs[0].pk, vrfs[1].rd]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(importing_vrfs__in=vrfs).distinct()
-        )
-
-    def test_exporting_vrfs(self):
-        vrfs = list(VRF.objects.filter(export_targets__isnull=False, rd__isnull=False).distinct())[:2]
-        params = {"exporting_vrfs": [vrfs[0].pk, vrfs[1].rd]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(exporting_vrfs__in=vrfs).distinct()
-        )
-
-    def test_search(self):
-        value = self.queryset.values_list("pk", flat=True)[0]
-        params = {"q": value}
-        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
-
-
-class RIRTestCase(FilterTestCases.NameOnlyFilterTestCase):
+class RIRTestCase(FilterTestCases.FilterTestCase):
     queryset = RIR.objects.all()
     filterset = RIRFilterSet
-
-    def test_description(self):
-        descriptions = self.queryset.exclude(description="").values_list("description", flat=True)[:2]
-        params = {"description": list(descriptions)}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+    generic_filter_tests = (
+        ("description",),
+        ("name",),
+    )
 
     def test_is_private(self):
         params = {"is_private": "true"}
@@ -159,16 +167,36 @@ class PrefixTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilt
     filterset = PrefixFilterSet
     tenancy_related_name = "prefixes"
     generic_filter_tests = (
+        ["cloud_networks", "cloud_networks__name"],
         ["date_allocated"],
         ["prefix_length"],
         ["rir", "rir__id"],
         ["rir", "rir__name"],
         ["role", "role__id"],
         ["role", "role__name"],
+        ["status", "status__id"],
         ["status", "status__name"],
         ["type"],
     )
 
+    def test_filters_generic(self):
+        # We usually have type container and network prefixes present
+        # but we need a third value for test_filters_generic() to work.
+        # So this is to ensure we have type pool prefixes in our test data.
+        Prefix.objects.create(
+            prefix="192.0.2.0/29",
+            type=PrefixTypeChoices.TYPE_POOL,
+            namespace=Namespace.objects.first(),
+            status=Status.objects.get_for_model(Prefix).first(),
+        )
+        Prefix.objects.create(
+            prefix="192.0.2.0/30",
+            type=PrefixTypeChoices.TYPE_POOL,
+            namespace=Namespace.objects.first(),
+            status=Status.objects.get_for_model(Prefix).first(),
+        )
+        return super().test_filters_generic()
+
     def test_search(self):
         prefixes = Prefix.objects.all()[:2]
         test_values = [
@@ -193,6 +221,47 @@ class PrefixTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilt
         self.assertQuerysetEqualAndNotEmpty(self.filterset(params, self.queryset).qs, all_prefixes)
 
 
+class PrefixLocationAssignmentTestCase(FilterTestCases.FilterTestCase):
+    queryset = PrefixLocationAssignment.objects.all()
+    filterset = PrefixLocationAssignmentFilterSet
+    # NOTE: No generic logic in place yet to test TreeNodeMultipleChoiceFilter
+    generic_filter_tests = ()
+    # generic_filter_tests = (
+    #     ["location", "location__name"],
+    #     ["location", "location__id"],
+    # )
+
+    def test_prefix(self):
+        ipv4_prefix = self.queryset.filter(prefix__ip_version=4).first().prefix
+        ipv6_prefix = self.queryset.filter(prefix__ip_version=6).first().prefix
+
+        params = {"prefix": [ipv4_prefix.prefix, ipv6_prefix.pk]}
+
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(prefix=ipv6_prefix)
+            | self.queryset.filter(
+                prefix__network=ipv4_prefix.network,
+                prefix__prefix_length=ipv4_prefix.prefix_length,
+                prefix__broadcast=ipv4_prefix.broadcast,
+            ),
+            ordered=False,
+        )
+
+        params = {"prefix": [ipv4_prefix.pk, ipv6_prefix.prefix]}
+
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(prefix=ipv4_prefix)
+            | self.queryset.filter(
+                prefix__network=ipv6_prefix.network,
+                prefix__prefix_length=ipv6_prefix.prefix_length,
+                prefix__broadcast=ipv6_prefix.broadcast,
+            ),
+            ordered=False,
+        )
+
+
 class PrefixFilterCustomDataTestCase(TestCase):
     """Filter test case that requires clearing out factory data and building specific test data."""
 
@@ -279,11 +348,21 @@ class PrefixFilterCustomDataTestCase(TestCase):
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs, self.queryset.filter(parent=parent4)
         )
+        params = {"parent": ["10.0.0.0/16"]}
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(parent__network="10.0.0.0", parent__prefix_length=16),
+        )
         parent6 = Prefix.objects.get(prefix="2001:db8::/32", namespace=self.namespace)
         params = {"parent": [str(parent6.pk)]}
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs, self.queryset.filter(parent=parent6)
         )
+        params = {"parent": ["2001:db8::/32"]}
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(parent__network="2001:db8::", parent__prefix_length=32),
+        )
 
     def test_ip_version(self):
         params = {"ip_version": "6"}
@@ -309,10 +388,9 @@ class PrefixFilterCustomDataTestCase(TestCase):
     def test_contains(self):
         matches_ipv4 = self.queryset.filter(network__in=["10.0.0.0", "10.0.1.0"])
         matches_ipv6 = self.queryset.filter(network__in=["2001:db8::", "2001:db8:0:1::"])
-        params = {"contains": "10.0.1.0/24"}
-        self.assertQuerysetEqualAndNotEmpty(self.filterset(params, self.queryset).qs, matches_ipv4)
-        params = {"contains": "2001:db8:0:1::/64"}
-        self.assertQuerysetEqualAndNotEmpty(self.filterset(params, self.queryset).qs, matches_ipv6)
+
+        params = {"contains": ["10.0.1.0/24", "2001:db8:0:1::"]}
+        self.assertQuerysetEqualAndNotEmpty(self.filterset(params, self.queryset).qs, matches_ipv4 | matches_ipv6)
 
     def test_vrfs(self):
         prefixes = self.queryset[:2]
@@ -383,20 +461,29 @@ class PrefixFilterCustomDataTestCase(TestCase):
         )
         test_locations[1].parent = test_locations[0]
         test_prefixes = list(self.queryset[:3])
-        test_prefixes[0].location = test_locations[0]
-        test_prefixes[1].location = test_locations[1]
-        test_prefixes[2].location = test_locations[2]
-        self.queryset.bulk_update(test_prefixes, ["location"])
+        test_prefixes[0].locations.set([test_locations[0]])
+        test_prefixes[1].locations.set([test_locations[1]])
+        test_prefixes[2].locations.set([test_locations[2]])
 
         params = {"location": [test_locations[0].pk, test_locations[1].pk]}
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs,
-            self.queryset.filter(location__in=params["location"]),
+            self.queryset.filter(locations__in=params["location"]),
         )
         params = {"location": [test_locations[0].name, test_locations[1].name]}
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs,
-            self.queryset.filter(location__name__in=params["location"]),
+            self.queryset.filter(locations__name__in=params["location"]),
+        )
+        params = {"locations": [test_locations[0].pk, test_locations[1].pk]}
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(locations__in=params["locations"]),
+        )
+        params = {"locations": [test_locations[0].name, test_locations[1].name]}
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(locations__name__in=params["locations"]),
         )
 
     def test_vlan(self):
@@ -421,20 +508,21 @@ class IPAddressTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyF
     queryset = IPAddress.objects.all()
     filterset = IPAddressFilterSet
     tenancy_related_name = "ip_addresses"
+    generic_filter_tests = (["nat_inside", "nat_inside__id"],)
 
     @classmethod
     def setUpTestData(cls):
         # Create some VRFs that belong to the same Namespace and have an rd
         cls.namespace = Namespace.objects.create(name="IP Address Test Case Namespace")
-        VRF.objects.create(name="VRF 1", rd="65000:100", namespace=cls.namespace)
-        VRF.objects.create(name="VRF 2", rd="65000:200", namespace=cls.namespace)
-        VRF.objects.create(name="VRF 3", rd="65000:300", namespace=cls.namespace)
+        vrfs = (
+            VRF.objects.create(name="VRF 1", rd="65000:100", namespace=cls.namespace),
+            VRF.objects.create(name="VRF 2", rd="65000:200", namespace=cls.namespace),
+            VRF.objects.create(name="VRF 3", rd="65000:300", namespace=cls.namespace),
+        )
         # Create some VRFs without an rd
         VRF.objects.create(name="VRF 4", namespace=cls.namespace)
         VRF.objects.create(name="VRF 5", namespace=cls.namespace)
         VRF.objects.create(name="VRF 6", namespace=cls.namespace)
-        vrfs = VRF.objects.filter(namespace=cls.namespace, rd__isnull=False)
-        assert len(vrfs) == 3, f"This Namespace {cls.namespace} does not contain enough VRFs."
 
         cls.interface_ct = ContentType.objects.get_for_model(Interface)
         cls.vm_interface_ct = ContentType.objects.get_for_model(VMInterface)
@@ -597,6 +685,20 @@ class IPAddressTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyF
             status=statuses[0],
             namespace=cls.namespace,
         )
+        IPAddress.objects.create(
+            address="10.1.1.1/32",
+            tenant=None,
+            status=statuses[0],
+            namespace=cls.namespace,
+            nat_inside=ip0,
+        )
+        IPAddress.objects.create(
+            address="10.2.2.2/32",
+            tenant=None,
+            status=statuses[0],
+            namespace=cls.namespace,
+            nat_inside=ip1,
+        )
 
     def test_search(self):
         ipv4_octets = self.ipv4_address.host.split(".")
@@ -663,14 +765,11 @@ class IPAddressTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyF
 
     def test_prefix(self):
         ipv4_parent = self.queryset.filter(ip_version=4).first().address.supernet()[-1]
-        params = {"prefix": str(ipv4_parent)}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs, self.queryset.net_host_contained(ipv4_parent)
-        )
         ipv6_parent = self.queryset.filter(ip_version=6).first().address.supernet()[-1]
-        params = {"prefix": str(ipv6_parent)}
+
+        params = {"prefix": [str(ipv4_parent), str(ipv6_parent)]}
         self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs, self.queryset.net_host_contained(ipv6_parent)
+            self.filterset(params, self.queryset).qs, self.queryset.net_host_contained(ipv4_parent, ipv6_parent)
         )
 
     def test_filter_address(self):
@@ -796,8 +895,8 @@ class IPAddressTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyF
 
     def test_present_in_vrf(self):
         # clear out all the randomly generated route targets and vrfs before running this custom test
-        test_ip_addresses_pk_list = list(self.queryset.values_list("pk", flat=True)[:10])
-        test_ip_addresses = self.queryset[:10]
+        test_ip_addresses = self.queryset.filter(parent__namespace=self.namespace)
+        test_ip_addresses_pk_list = list(test_ip_addresses.values_list("pk", flat=True))
         # With advent of `IPAddress.parent`, IPAddresses can't just be bulk deleted without clearing their
         # `parent` first in an `update()` query which doesn't call `save()` or `fire `(pre|post)_save` signals.
         unwanted_ips = self.queryset.exclude(pk__in=test_ip_addresses_pk_list)
@@ -815,12 +914,6 @@ class IPAddressTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyF
             VRF.objects.create(name="VRF 2", rd="65000:200", namespace=self.namespace),
             VRF.objects.create(name="VRF 3", rd="65000:300", namespace=self.namespace),
         )
-        test_ip_addresses[0].parent.namespace = self.namespace
-        test_ip_addresses[0].validated_save()
-        test_ip_addresses[1].parent.namespace = self.namespace
-        test_ip_addresses[1].validated_save()
-        test_ip_addresses[2].parent.namespace = self.namespace
-        test_ip_addresses[2].validated_save()
         vrfs[0].import_targets.add(route_targets[0], route_targets[1], route_targets[2])
         vrfs[1].export_targets.add(route_targets[1])
         vrfs[2].export_targets.add(route_targets[2])
@@ -855,7 +948,7 @@ class IPAddressTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyF
 
     def test_status(self):
         statuses = list(Status.objects.get_for_model(IPAddress).filter(ip_addresses__isnull=False)[:2])
-        params = {"status": [statuses[0].name, statuses[1].name]}
+        params = {"status": [statuses[0].name, statuses[1].id]}
         self.assertQuerysetEqualAndNotEmpty(
             self.filterset(params, self.queryset).qs, self.queryset.filter(status__in=statuses)
         )
@@ -907,9 +1000,14 @@ class IPAddressToInterfaceTestCase(FilterTestCases.FilterTestCase):
         vm_status = Status.objects.get_for_model(VirtualMachine).first()
         virtual_machine = (VirtualMachine.objects.create(name="Virtual Machine 1", cluster=cluster, status=vm_status),)
         vm_int_status = Status.objects.get_for_model(VMInterface).first()
+        vm_int_role = Role.objects.get_for_model(VMInterface).first()
         vm_interfaces = [
-            VMInterface.objects.create(virtual_machine=virtual_machine[0], name="veth0", status=vm_int_status),
-            VMInterface.objects.create(virtual_machine=virtual_machine[0], name="veth1", status=vm_int_status),
+            VMInterface.objects.create(
+                virtual_machine=virtual_machine[0], name="veth0", status=vm_int_status, role=vm_int_role
+            ),
+            VMInterface.objects.create(
+                virtual_machine=virtual_machine[0], name="veth1", status=vm_int_status, role=vm_int_role
+            ),
         ]
 
         IPAddressToInterface.objects.create(ip_address=ip_addresses[0], interface=interfaces[0], vm_interface=None)
@@ -951,9 +1049,77 @@ class IPAddressToInterfaceTestCase(FilterTestCases.FilterTestCase):
                 )
 
 
-class VLANGroupTestCase(FilterTestCases.NameOnlyFilterTestCase):
+class VRFDeviceAssignmentTestCase(FilterTestCases.FilterTestCase):
+    queryset = VRFDeviceAssignment.objects.all()
+    filterset = VRFDeviceAssignmentFilterSet
+    generic_filter_tests = (
+        ["vrf", "vrf__id"],
+        ["vrf", "vrf__name"],
+        ["device", "device__id"],
+        ["device", "device__name"],
+        ["virtual_machine", "virtual_machine__id"],
+        ["virtual_machine", "virtual_machine__name"],
+        ["virtual_device_context", "virtual_device_context__id"],
+        ["virtual_device_context", "virtual_device_context__name"],
+        ["name"],
+        ["rd"],
+    )
+
+    @classmethod
+    def setUpTestData(cls):
+        # Creating VRFDeviceAssignment instances manually until VRFFactory is enhanced to generate VRFDeviceAssignments
+        cls.vrfs = VRF.objects.all()
+        cls.devices = Device.objects.all()
+        cls.vdcs = VirtualDeviceContext.objects.all()
+        locations = Location.objects.filter(location_type__name="Campus")
+        cluster_type = ClusterType.objects.create(name="Test Cluster Type")
+        cluster = Cluster.objects.create(name="Cluster 1", cluster_type=cluster_type, location=locations[0])
+        vm_status = Status.objects.get_for_model(VirtualMachine).first()
+        vm_role = Role.objects.get_for_model(VirtualMachine).first()
+        cls.test_vm_1 = VirtualMachine.objects.create(
+            cluster=cluster,
+            name="VM 1",
+            role=vm_role,
+            status=vm_status,
+        )
+        cls.test_vm_2 = VirtualMachine.objects.create(
+            cluster=cluster,
+            name="VM 2",
+            role=vm_role,
+            status=vm_status,
+        )
+        VRFDeviceAssignment.objects.create(
+            vrf=cls.vrfs[0],
+            device=cls.devices[0],
+            rd="65000:1",
+        )
+        VRFDeviceAssignment.objects.create(
+            vrf=cls.vrfs[0],
+            device=cls.devices[1],
+            rd="65000:2",
+        )
+        VRFDeviceAssignment.objects.create(
+            vrf=cls.vrfs[0],
+            virtual_machine=cls.test_vm_1,
+            rd="65000:3",
+        )
+        VRFDeviceAssignment.objects.create(
+            vrf=cls.vrfs[1],
+            virtual_machine=cls.test_vm_2,
+            rd="65000:4",
+        )
+
+
+class VLANGroupTestCase(FilterTestCases.FilterTestCase):
     queryset = VLANGroup.objects.all()
     filterset = VLANGroupFilterSet
+    generic_filter_tests = (
+        ("description",),
+        # NOTE: No generic logic in place yet to test TreeNodeMultipleChoiceFilter
+        # ("location", "location__id"),
+        # ("location", "location__name"),
+        ("name",),
+    )
 
     @classmethod
     def setUpTestData(cls):
@@ -973,31 +1139,26 @@ class VLANGroupTestCase(FilterTestCases.NameOnlyFilterTestCase):
         VLANGroup.objects.create(name="VLAN Group 3", location=cls.locations[2], description="C")
         VLANGroup.objects.create(name="VLAN Group 4", location=None)
 
-    def test_description(self):
-        descriptions = list(VLANGroup.objects.exclude(description="").values_list("description", flat=True)[:2])
-        params = {"description": descriptions}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_location(self):
-        params = {"location": [self.locations[0].pk, self.locations[1].pk]}
-        self.assertQuerysetEqual(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(location__in=params["location"])
-        )
-        params = {"location": [self.locations[0].name, self.locations[1].name]}
-        self.assertQuerysetEqual(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(location__name__in=params["location"])
-        )
-
 
 class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilterTestCaseMixin):
     queryset = VLAN.objects.all()
     filterset = VLANFilterSet
     tenancy_related_name = "vlans"
+    generic_filter_tests = (
+        ("name",),
+        ("role", "role__id"),
+        ("role", "role__name"),
+        ("status", "status__id"),
+        ("status", "status__name"),
+        ("vid",),
+        ("vlan_group", "vlan_group__id"),
+        ("vlan_group", "vlan_group__name"),
+    )
 
     @classmethod
     def setUpTestData(cls):
         cls.location_type_1 = LocationType.objects.get(name="Campus")
-        cls.location_type_2 = LocationType.objects.get(name="Building")
+        cls.location_type_2 = LocationType.objects.get(name="Aisle")
         loc_status = Status.objects.get_for_model(Location).first()
         cls.locations = (
             Location.objects.create(name="Location 1", location_type=cls.location_type_1, status=loc_status),
@@ -1022,8 +1183,8 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
             VLAN.objects.create(
                 vid=101,
                 name="VLAN 101",
-                location=cls.locations[0],
                 vlan_group=groups[0],
+                location=cls.locations[0],
                 role=roles[0],
                 tenant=tenants[0],
                 status=statuses[0],
@@ -1031,7 +1192,6 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
             VLAN.objects.create(
                 vid=102,
                 name="VLAN 102",
-                location=cls.locations[0],
                 vlan_group=groups[0],
                 role=roles[0],
                 tenant=tenants[0],
@@ -1040,7 +1200,6 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
             VLAN.objects.create(
                 vid=201,
                 name="VLAN 201",
-                location=cls.locations[1],
                 vlan_group=groups[1],
                 role=roles[1],
                 tenant=tenants[1],
@@ -1049,7 +1208,6 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
             VLAN.objects.create(
                 vid=202,
                 name="VLAN 202",
-                location=cls.locations[1],
                 vlan_group=groups[1],
                 role=roles[1],
                 tenant=tenants[1],
@@ -1058,7 +1216,6 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
             VLAN.objects.create(
                 vid=301,
                 name="VLAN 301",
-                location=cls.locations[2],
                 vlan_group=groups[2],
                 role=roles[2],
                 tenant=tenants[2],
@@ -1067,60 +1224,42 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
             VLAN.objects.create(
                 vid=302,
                 name="VLAN 302",
-                location=cls.locations[2],
                 vlan_group=groups[2],
                 role=roles[2],
                 tenant=tenants[2],
                 status=statuses[2],
             ),
         )
+        vlans[1].locations.add(cls.locations[0])
+        vlans[2].locations.add(cls.locations[1])
+        vlans[3].locations.add(cls.locations[1])
+        vlans[4].locations.set([cls.locations[1], cls.locations[2]])
+        vlans[5].locations.set([cls.locations[0], cls.locations[2]])
+
         vlans[0].tags.set(Tag.objects.all()[:2])
         vlans[1].tags.set(Tag.objects.all()[:2])
 
-    def test_name(self):
-        names = list(VLAN.objects.all().values_list("name", flat=True)[:2])
-        params = {"name": names}
-        self.assertQuerysetEqual(self.filterset(params, self.queryset).qs, self.queryset.filter(name__in=names))
-
-    def test_vid(self):
-        vids = list(VLAN.objects.all().values_list("vid", flat=True)[:3])
-        params = {"vid": vids}
-        self.assertQuerysetEqual(self.filterset(params, self.queryset).qs, self.queryset.filter(vid__in=vids))
-
     def test_location(self):
         params = {"location": [self.locations[0].pk, self.locations[1].pk]}
         self.assertQuerysetEqual(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(location__in=params["location"])
+            self.filterset(params, self.queryset).qs, self.queryset.filter(locations__in=params["location"]).distinct()
         )
         params = {"location": [self.locations[0].name, self.locations[1].name]}
         self.assertQuerysetEqual(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(location__name__in=params["location"])
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(locations__name__in=params["location"]).distinct(),
         )
 
-    def test_vlan_group(self):
-        groups = list(VLANGroup.objects.filter(vlans__isnull=False).distinct())[:2]
-        filter_params = [{"vlan_group": [groups[0].pk, groups[1].pk]}, {"vlan_group": [groups[0].pk, groups[1].name]}]
-        for params in filter_params:
-            self.assertQuerysetEqualAndNotEmpty(
-                self.filterset(params, self.queryset).qs, self.queryset.filter(vlan_group__in=groups)
-            )
-
-    def test_role(self):
-        roles = Role.objects.get_for_model(VLAN)[:2]
-        params = {"role": [roles[0].pk, roles[1].name]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(role__in=[roles[0], roles[1]])
+    def test_locations(self):
+        params = {"locations": [self.locations[0].pk, self.locations[1].pk]}
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs, self.queryset.filter(locations__in=params["locations"]).distinct()
+        )
+        params = {"locations": [self.locations[0].name, self.locations[1].name]}
+        self.assertQuerysetEqual(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(locations__name__in=params["locations"]).distinct(),
         )
-
-    def test_status(self):
-        statuses = list(Status.objects.get_for_model(VLAN).filter(vlans__isnull=False).distinct())[:2]
-        params = {"status": [statuses[0].name, statuses[1].name]}
-        self.assertQuerysetEqual(self.filterset(params, self.queryset).qs, self.queryset.filter(status__in=statuses))
-
-    def test_search(self):
-        value = self.queryset.values_list("pk", flat=True)[0]
-        params = {"q": value}
-        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
     def test_available_on_device(self):
         manufacturer = Manufacturer.objects.first()
@@ -1134,10 +1273,48 @@ class VLANTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilter
         params = {"available_on_device": [device.pk]}
         self.assertQuerysetEqual(
             self.filterset(params, self.queryset).qs,
-            self.queryset.filter(Q(location=device.location) | Q(location__isnull=True)),
+            self.queryset.filter(Q(locations__in=[device.location]) | Q(locations__isnull=True)),
         )
 
 
+class VLANLocationAssignmentTestCase(FilterTestCases.FilterTestCase):
+    queryset = VLANLocationAssignment.objects.all()
+    filterset = VLANLocationAssignmentFilterSet
+
+    generic_filter_tests = (
+        ["vlan", "vlan__vid"],
+        # NOTE: No generic logic in place yet to test TreeNodeMultipleChoiceFilter
+        # ["location", "location__name"],
+        # ["location", "location__id"],
+    )
+
+    def test_q_filter_vlan__vid_predicate(self):
+        vlan = VLAN.objects.first()
+        vlan_ct = ContentType.objects.get_for_model(vlan)
+        vlan_vid = vlan.vid
+
+        # Exclude any locations that might cause conflicts with our FilterSet TestCase,
+        # the 'VLANLocationAssignmentFilterSet' `q` field filters
+        # based on both `vlan__vid` and `location__name`. We want to ensure that
+        # the location names do not contain the VLAN ID to avoid collisions.
+        vlan_location_queryset = VLANLocationAssignment.objects.filter(vlan__vid__exact=vlan_vid).exclude(
+            location__name__icontains=vlan_vid
+        )
+        # If we don't have enough queryset items for testing (less than 5),
+        # add 5 VLANLocationAssignments to test on.
+        if vlan_location_queryset.count() < 5:
+            locations = Location.objects.filter(location_type__content_types__in=[vlan_ct]).exclude(
+                name__icontains=vlan_vid
+            )[:5]
+            vlan.locations.set(locations)
+
+        params = {"q": vlan_vid}
+        queryset = VLANLocationAssignment.objects.exclude(location__name__icontains=vlan_vid)
+        filterset = VLANLocationAssignmentFilterSet(params, queryset).qs
+        expected_queryset = queryset.filter(vlan__vid__exact=vlan_vid)
+        self.assertQuerysetEqualAndNotEmpty(filterset, expected_queryset)
+
+
 class ServiceTestCase(FilterTestCases.FilterTestCase):
     queryset = Service.objects.all()
     filterset = ServiceFilterSet
@@ -1239,8 +1416,3 @@ class ServiceTestCase(FilterTestCases.FilterTestCase):
     def test_ports(self):
         params = {"ports": "1001"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-    def test_search(self):
-        value = self.queryset.values_list("pk", flat=True)[0]
-        params = {"q": value}
-        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
