@@ -203,6 +203,10 @@ class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.List
             "tags": [t.pk for t in Tag.objects.get_for_model(Prefix)],
         }
 
+        cls.update_data = cls.form_data.copy()
+        # Can't update `prefix` and `namespace` in the same edit request
+        cls.update_data["namespace"] = Prefix.objects.first().namespace.pk
+
         cls.bulk_edit_data = {
             "tenant": None,
             "status": cls.statuses[1].pk,
@@ -217,6 +221,22 @@ class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.List
             "remove_vrfs": [vrfs[1].pk],
         }
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_get_object_with_permission(self):
+        response = super().test_get_object_with_permission()
+
+        content = extract_page_body(response.content.decode(response.charset))
+        self.assertNotIn("The parent field on this record appears to be set incorrectly", strip_tags(content))
+
+        instance = self._get_queryset().first()
+        instance.parent = self._get_queryset().last()
+        self._get_queryset().bulk_update([instance], ["parent"], batch_size=1)
+
+        response = super().test_get_object_with_permission()
+
+        content = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("The parent field on this record appears to be set incorrectly", strip_tags(content))
+
     @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
     def test_list_objects_with_permission(self):
         """Test rendering of LinkedCountColumn for related fields with display_field override."""
@@ -258,133 +278,6 @@ class PrefixTestCase(ViewTestCases.PrimaryObjectViewTestCase, ViewTestCases.List
         for prefix in prefixes:
             self.assertNotIn(prefix.get_absolute_url(), content, msg=content)
 
-    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
-    def test_create_object_warnings(self):
-        """Test various object creation scenarios that should result in a warning to the user."""
-        Prefix.objects.create(
-            prefix="10.0.0.0/8",
-            namespace=self.namespace,
-            type=PrefixTypeChoices.TYPE_CONTAINER,
-            status=self.statuses[1],
-        )
-        Prefix.objects.create(
-            prefix="10.0.0.0/16",
-            namespace=self.namespace,
-            type=PrefixTypeChoices.TYPE_NETWORK,
-            status=self.statuses[1],
-        )
-        Prefix.objects.create(
-            prefix="10.0.0.0/24",
-            namespace=self.namespace,
-            type=PrefixTypeChoices.TYPE_POOL,
-            status=self.statuses[1],
-        )
-        IPAddress.objects.create(
-            address="10.0.0.1/32",
-            status=Status.objects.get_for_model(IPAddress).first(),
-            namespace=self.namespace,
-        )
-        self.add_permissions("ipam.add_prefix")
-
-        common_data = {"namespace": self.namespace.pk, "status": self.statuses[0].pk}
-
-        with self.subTest("Creating a Pool as child of a Container raises a warning"):
-            data = {
-                "prefix": "10.1.0.0/16",
-                "type": PrefixTypeChoices.TYPE_POOL,
-            }
-            response = self.client.post(self._get_url("add"), data={**common_data, **data}, follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "10.1.0.0/16 is a Pool prefix but its parent 10.0.0.0/8 is a Container. "
-                "This will be considered invalid data in a future release. "
-                "Consider changing the type of 10.1.0.0/16 and/or 10.0.0.0/8 to resolve this issue.",
-                strip_tags(content),
-            )
-
-        # We could test for Pool-in-Pool, Container-in-Network, Network-in-Network, Container-in-Pool, and
-        # Network-in-Pool, but they all use the same code path and similar message
-
-        with self.subTest("Creating a Container that will have a Pool as its child raises a warning"):
-            data = {
-                "prefix": "10.0.0.0/20",
-                "type": PrefixTypeChoices.TYPE_CONTAINER,
-            }
-            response = self.client.post(self._get_url("add"), data={**common_data, **data}, follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "10.0.0.0/20 is a Container prefix and should not contain child prefixes of type Pool. "
-                "This will be considered invalid data in a future release. "
-                "Consider creating an intermediary Network prefix, or changing the type of its children to Network, "
-                "to resolve this issue.",
-                strip_tags(content),
-            )
-
-        with self.subTest("Creating a Network that will have another Network as its child raises a warning"):
-            data = {
-                "prefix": "10.0.0.0/12",
-                "type": PrefixTypeChoices.TYPE_NETWORK,
-            }
-            response = self.client.post(self._get_url("add"), data={**common_data, **data}, follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "10.0.0.0/12 is a Network prefix and should not contain child prefixes of types Container or Network. "
-                "This will be considered invalid data in a future release. "
-                "Consider changing the type of 10.0.0.0/12 to Container, or changing the type of its children to Pool, "
-                "to resolve this issue.",
-                strip_tags(content),
-            )
-
-        with self.subTest("Creating a Pool that will have any other Prefix as its child raises a warning"):
-            data = {
-                "prefix": "0.0.0.0/0",
-                "type": PrefixTypeChoices.TYPE_POOL,
-            }
-            response = self.client.post(self._get_url("add"), data={**common_data, **data}, follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "0.0.0.0/0 is a Pool prefix and should not contain other prefixes. "
-                "This will be considered invalid data in a future release. "
-                "Consider either changing the type of 0.0.0.0/0 to Container or Network, or deleting its children, "
-                "to resolve this issue.",
-                strip_tags(content),
-            )
-
-        with self.subTest("Creating a large Container that will contain IPs raises a warning"):
-            data = {
-                "prefix": "10.0.0.0/28",
-                "type": PrefixTypeChoices.TYPE_CONTAINER,
-            }
-            response = self.client.post(self._get_url("add"), data={**common_data, **data}, follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "10.0.0.0/28 is a Container prefix and should not directly contain IP addresses. "
-                "This will be considered invalid data in a future release. "
-                "Consider either changing the type of 10.0.0.0/28 to Network, or creating one or more child "
-                "prefix(es) of type Network to contain these IP addresses, to resolve this issue.",
-                strip_tags(content),
-            )
-
-        with self.subTest("Creating a small Container that will contain IPs raises a different warning"):
-            data = {
-                "prefix": "10.0.0.1/32",
-                "type": PrefixTypeChoices.TYPE_CONTAINER,
-            }
-            response = self.client.post(self._get_url("add"), data={**common_data, **data}, follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "10.0.0.1/32 is a Container prefix and should not directly contain IP addresses. "
-                "This will be considered invalid data in a future release. "
-                "Consider changing the type of 10.0.0.1/32 to Network to resolve this issue.",
-                strip_tags(content),
-            )
-
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_prefix_ipaddresses_table_list_includes_child_ips(self):
         ip_status = Status.objects.get_for_model(IPAddress).first()
@@ -432,7 +325,7 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
         cls.statuses = Status.objects.get_for_model(IPAddress)
         cls.prefix_status = Status.objects.get_for_model(Prefix).first()
         roles = Role.objects.get_for_model(IPAddress)
-        Prefix.objects.get_or_create(
+        cls.prefix, _ = Prefix.objects.get_or_create(
             prefix="192.0.2.0/24",
             defaults={"namespace": cls.namespace, "status": cls.prefix_status, "type": "network"},
         )
@@ -459,6 +352,22 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "description": "New description",
         }
 
+    @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
+    def test_get_object_with_permission(self):
+        response = super().test_get_object_with_permission()
+
+        content = extract_page_body(response.content.decode(response.charset))
+        self.assertNotIn("The parent field on this record appears to be set incorrectly", strip_tags(content))
+
+        instance = self._get_queryset().first()
+        instance.parent = self.prefix
+        self._get_queryset().bulk_update([instance], ["parent"], batch_size=1)
+
+        response = super().test_get_object_with_permission()
+
+        content = extract_page_body(response.content.decode(response.charset))
+        self.assertIn("The parent field on this record appears to be set incorrectly", strip_tags(content))
+
     def test_edit_object_with_permission(self):
         instance = self._get_queryset().first()
         form_data = self.form_data.copy()
@@ -516,7 +425,9 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "data": post_data(form_data),
         }
         response = self.client.post(**request)
-        self.assertBodyContains(response, "No suitable parent Prefix exists in this Namespace")
+        self.assertBodyContains(
+            response, f"No suitable parent Prefix for {instance.host} exists in Namespace {new_namespace}"
+        )
         # Create an exact copy of the parent prefix but in a different namespace. See if the re-parenting is successful
         new_parent = Prefix.objects.create(
             prefix=instance.parent.prefix,
@@ -529,41 +440,6 @@ class IPAddressTestCase(ViewTestCases.PrimaryObjectViewTestCase):
         created_ip = IPAddress.objects.get(parent__namespace=new_namespace, address=instance.address)
         self.assertEqual(created_ip.parent, new_parent)
 
-    @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
-    def test_create_object_warnings(self):
-        self.add_permissions("ipam.add_ipaddress")
-
-        Prefix.objects.create(
-            prefix="192.0.2.0/25",
-            namespace=self.namespace,
-            type=PrefixTypeChoices.TYPE_CONTAINER,
-            status=self.prefix_status,
-        )
-
-        with self.subTest("Creating an IPAddress as a child of a larger Container prefix raises a warning"):
-            self.form_data["address"] = "192.0.2.98/28"
-            response = self.client.post(self._get_url("add"), data=post_data(self.form_data), follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "IP address 192.0.2.98/28 currently has prefix 192.0.2.0/25 as its parent, which is a Container. "
-                "This will be considered invalid data in a future release. "
-                "Consider creating an intermediate /28 prefix of type Network to resolve this issue.",
-                strip_tags(content),
-            )
-
-        with self.subTest("Creating an IP as a child of a same-size Container prefix raises a different warning"):
-            self.form_data["address"] = "192.0.2.2/25"
-            response = self.client.post(self._get_url("add"), data=post_data(self.form_data), follow=True)
-            self.assertHttpStatus(response, 200)
-            content = extract_page_body(response.content.decode(response.charset))
-            self.assertIn(
-                "IP address 192.0.2.2/25 currently has prefix 192.0.2.0/25 as its parent, which is a Container. "
-                "This will be considered invalid data in a future release. "
-                "Consider changing the prefix to type Network or Pool to resolve this issue.",
-                strip_tags(content),
-            )
-
     @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
     def test_bulk_create_ips(self):
         """"""
