@@ -1,9 +1,8 @@
 from nautobot.core.api.routers import OrderedDefaultRouter
+
 from . import views
 
-
-router = OrderedDefaultRouter()
-router.APIRootView = views.TenancyRootView
+router = OrderedDefaultRouter(view_name="Tenancy")
 
 # Tenants
 router.register("tenant-groups", views.TenantGroupViewSet)
