@@ -1,7 +1,7 @@
 import django_filters
 
+from nautobot.core.filters import NaturalKeyOrPKMultipleChoiceFilter, TreeNodeMultipleChoiceFilter
 from nautobot.tenancy.models import Tenant, TenantGroup
-from nautobot.utilities.filters import NaturalKeyOrPKMultipleChoiceFilter, TreeNodeMultipleChoiceFilter
 
 
 class TenancyModelFilterSetMixin(django_filters.FilterSet):
@@ -9,16 +9,11 @@ class TenancyModelFilterSetMixin(django_filters.FilterSet):
     An inheritable FilterSet for models which support Tenant assignment.
     """
 
-    tenant_group_id = TreeNodeMultipleChoiceFilter(
-        queryset=TenantGroup.objects.all(),
-        field_name="tenant__group",
-        label="Tenant Group (ID)",
-    )
     tenant_group = TreeNodeMultipleChoiceFilter(
         queryset=TenantGroup.objects.all(),
-        field_name="tenant__group",
-        to_field_name="slug",
-        label="Tenant Group (slug)",
+        field_name="tenant__tenant_group",
+        to_field_name="name",
+        label="Tenant Group (name or ID)",
     )
     tenant_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Tenant.objects.all(),
@@ -26,5 +21,6 @@ class TenancyModelFilterSetMixin(django_filters.FilterSet):
     )
     tenant = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=Tenant.objects.all(),
-        label="Tenant (slug or ID)",
+        to_field_name="name",
+        label="Tenant (name or ID)",
     )
