@@ -3,8 +3,9 @@ from nautobot.extras.models import Tag
 from nautobot.tenancy.models import Tenant, TenantGroup
 
 
-class TenantGroupTestCase(ViewTestCases.OrganizationalObjectViewTestCase):
+class TenantGroupTestCase(ViewTestCases.OrganizationalObjectViewTestCase, ViewTestCases.BulkEditObjectsViewTestCase):
     model = TenantGroup
+    sort_on_field = "name"
 
     @classmethod
     def setUpTestData(cls):
@@ -13,13 +14,9 @@ class TenantGroupTestCase(ViewTestCases.OrganizationalObjectViewTestCase):
             "description": "A new tenant group",
         }
 
-        cls.csv_data = (
-            "name,description",
-            "Tenant Group 4,Fourth tenant group",
-            "Tenant Group 5,Fifth tenant group",
-            "Tenant Group 6,Sixth tenant group",
-            "Tenant Group 7,Seventh tenant group",
-        )
+        cls.bulk_edit_data = {
+            "description": "New description",
+        }
 
 
 class TenantTestCase(ViewTestCases.PrimaryObjectViewTestCase):
@@ -38,14 +35,6 @@ class TenantTestCase(ViewTestCases.PrimaryObjectViewTestCase):
             "tags": [t.pk for t in Tag.objects.get_for_model(Tenant)],
         }
 
-        cls.csv_data = (
-            "name,description",
-            "Tenant 4,A tenant",
-            "Tenant 5,A tenant",
-            "Tenant 6,A tenant",
-            "Tenant 7,A tenant",
-        )
-
         cls.bulk_edit_data = {
             "tenant_group": tenant_groups[1].pk,
         }
