@@ -1,16 +1,14 @@
-from django.db.models import Q
-
 from nautobot.core.testing import FilterTestCases
 from nautobot.dcim.choices import InterfaceModeChoices
-from nautobot.dcim.models import Device, DeviceType, Location, LocationType, Manufacturer, Platform
+from nautobot.dcim.models import Device, DeviceType, Location, LocationType, Manufacturer, Platform, SoftwareVersion
 from nautobot.extras.models import Role, Status, Tag
 from nautobot.ipam.choices import ServiceProtocolChoices
-from nautobot.ipam.models import IPAddress, VLAN, Service, Namespace, Prefix
+from nautobot.ipam.models import IPAddress, Namespace, Prefix, Service, VLAN
 from nautobot.tenancy.models import Tenant
 from nautobot.virtualization.filters import (
-    ClusterTypeFilterSet,
-    ClusterGroupFilterSet,
     ClusterFilterSet,
+    ClusterGroupFilterSet,
+    ClusterTypeFilterSet,
     VirtualMachineFilterSet,
     VMInterfaceFilterSet,
 )
@@ -23,9 +21,13 @@ from nautobot.virtualization.models import (
 )
 
 
-class ClusterTypeTestCase(FilterTestCases.NameOnlyFilterTestCase):
+class ClusterTypeTestCase(FilterTestCases.FilterTestCase):
     queryset = ClusterType.objects.all()
     filterset = ClusterTypeFilterSet
+    generic_filter_tests = (
+        ("description",),
+        ("name",),
+    )
 
     @classmethod
     def setUpTestData(cls):
@@ -40,26 +42,19 @@ class ClusterTypeTestCase(FilterTestCases.NameOnlyFilterTestCase):
             Cluster.objects.create(name="Cluster 2", cluster_type=cluster_types[1]),
         ]
 
-    def test_description(self):
-        params = {"description": ["A", "B"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_clusters(self):
         with self.subTest("Clusters"):
             params = {"clusters": [self.clusters[0].pk, self.clusters[1].name]}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
-        with self.subTest("Has Clusters"):
-            params = {"has_clusters": True}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
-            params = {"has_clusters": False}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-
-class ClusterGroupTestCase(FilterTestCases.NameOnlyFilterTestCase):
+class ClusterGroupTestCase(FilterTestCases.FilterTestCase):
     queryset = ClusterGroup.objects.all()
     filterset = ClusterGroupFilterSet
+    generic_filter_tests = (
+        ("description",),
+        ("name",),
+    )
 
     @classmethod
     def setUpTestData(cls):
@@ -80,27 +75,32 @@ class ClusterGroupTestCase(FilterTestCases.NameOnlyFilterTestCase):
             Cluster.objects.create(name="Cluster 2", cluster_type=cluster_types[1], cluster_group=cluster_groups[1]),
         )
 
-    def test_description(self):
-        params = {"description": ["A", "B"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_clusters(self):
         with self.subTest("Clusters"):
             params = {"clusters": [self.clusters[0].pk, self.clusters[1].name]}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
-        with self.subTest("Has Clusters"):
-            params = {"has_clusters": True}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-            params = {"has_clusters": False}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
 
 class ClusterTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilterTestCaseMixin):
     queryset = Cluster.objects.all()
     filterset = ClusterFilterSet
     tenancy_related_name = "clusters"
+    generic_filter_tests = (
+        ("cluster_group", "cluster_group__id"),
+        ("cluster_group", "cluster_group__name"),
+        ("cluster_group_id", "cluster_group__id"),
+        ("cluster_type", "cluster_type__id"),
+        ("cluster_type", "cluster_type__name"),
+        ("cluster_type_id", "cluster_type__id"),
+        ("comments",),
+        # ("devices", "devices__id"),  # TODO: not enough distinct devices at present
+        # ("devices", "devices__name"),  # TODO: not enough distinct devices at present
+        # ("location", "location__id"),  # TODO
+        # ("location", "location__name"),  # TODO
+        ("name",),
+        # ("virtual_machines", "virtual_machines__id"),  # TODO: not enough distinct VMs at present
+        # ("virtual_machines", "virtual_machines__name"),  # TODO: not enough distinct VMs at present
+    )
 
     @classmethod
     def setUpTestData(cls):
@@ -179,38 +179,16 @@ class ClusterTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFil
         clusters[0].tags.set(Tag.objects.get_for_model(Cluster))
         clusters[1].tags.set(Tag.objects.get_for_model(Cluster)[:3])
 
-    def test_name(self):
-        params = {"name": ["Cluster 1", "Cluster 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_comments(self):
-        params = {"comments": ["This is cluster 1", "This is cluster 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_device(self):
         with self.subTest("Devices"):
             params = {"devices": [self.device.pk, self.device.name]}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
-        with self.subTest("Has Devices"):
-            params = {"has_devices": True}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-            params = {"has_devices": False}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_virtual_machines(self):
         with self.subTest("Virtual Machines"):
             params = {"virtual_machines": [self.virtualmachine.pk, self.virtualmachine.name]}
             self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
-        with self.subTest("Has Virtual Machines"):
-            params = {"has_virtual_machines": True}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-            params = {"has_virtual_machines": False}
-            self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_location(self):
         params = {"location": [self.locations[0].pk, self.locations[1].pk]}
         self.assertQuerysetEqual(
@@ -221,41 +199,47 @@ class ClusterTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFil
             self.filterset(params, self.queryset).qs, self.queryset.filter(location__name__in=params["location"])
         )
 
-    def test_cluster_group(self):
-        cluster_groups = list(ClusterGroup.objects.all()[:2])
-        filter_params = [
-            {"cluster_group_id": [cluster_groups[0].pk, cluster_groups[1].pk]},
-            {"cluster_group": [cluster_groups[0].pk, cluster_groups[1].name]},
-        ]
-        for params in filter_params:
-            self.assertQuerysetEqualAndNotEmpty(
-                self.filterset(params, self.queryset).qs,
-                self.queryset.filter(cluster_group__in=cluster_groups).distinct(),
-            )
-
-    def test_cluster_type(self):
-        cluster_types = list(ClusterType.objects.all()[:2])
-        filter_params = [
-            {"cluster_type_id": [cluster_types[0].pk, cluster_types[1].pk]},
-            {"cluster_type": [cluster_types[0].pk, cluster_types[1].name]},
-        ]
-        for params in filter_params:
-            self.assertQuerysetEqualAndNotEmpty(
-                self.filterset(params, self.queryset).qs,
-                self.queryset.filter(cluster_type__in=cluster_types).distinct(),
-            )
-
-    def test_search(self):
-        value = self.queryset.values_list("pk", flat=True)[0]
-        params = {"q": value}
-        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
-
 
 class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.TenancyFilterTestCaseMixin):
     queryset = VirtualMachine.objects.all()
     filterset = VirtualMachineFilterSet
     tenancy_related_name = "virtual_machines"
 
+    generic_filter_tests = (
+        ["ip_addresses", "interfaces__ip_addresses__id"],
+        ("cluster", "cluster__id"),
+        ("cluster", "cluster__name"),
+        ("cluster_group", "cluster__cluster_group__id"),
+        ("cluster_group", "cluster__cluster_group__name"),
+        ("cluster_group_id", "cluster__cluster_group__id"),
+        ("cluster_id", "cluster__id"),
+        ("cluster_type", "cluster__cluster_type__id"),
+        ("cluster_type", "cluster__cluster_type__name"),
+        ("cluster_type_id", "cluster__cluster_type__id"),
+        ("comments",),
+        ("disk",),
+        ("interfaces", "interfaces__id"),
+        ("location", "cluster__location__id"),
+        ("location", "cluster__location__name"),
+        ("mac_address", "interfaces__mac_address"),
+        ("memory",),
+        ("name",),
+        ("platform", "platform__id"),
+        ("platform", "platform__name"),
+        ("platform_id", "platform__id"),
+        ("role", "role__id"),
+        ("role", "role__name"),
+        ("services", "services__id"),
+        ("services", "services__name"),
+        ("software_image_files", "software_image_files__id"),
+        ("software_image_files", "software_image_files__image_file_name"),
+        ("software_version", "software_version__id"),
+        ("software_version", "software_version__version"),
+        ("status", "status__id"),
+        ("status", "status__name"),
+        ("vcpus",),
+    )
+
     @classmethod
     def setUpTestData(cls):
         cluster_types = (
@@ -309,6 +293,8 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
         roles = Role.objects.get_for_model(VirtualMachine)
         cls.roles = roles
 
+        cls.software_versions = SoftwareVersion.objects.filter(software_image_files__isnull=False)[:3]
+
         tenants = Tenant.objects.filter(tenant_group__isnull=False)[:3]
 
         cls.statuses = Status.objects.get_for_model(VirtualMachine)
@@ -326,6 +312,7 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
                 disk=1,
                 local_config_context_data={"foo": 123},
                 comments="This is VM 1",
+                software_version=cls.software_versions[0],
             ),
             VirtualMachine.objects.create(
                 name="Virtual Machine 2",
@@ -338,6 +325,7 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
                 memory=2,
                 disk=2,
                 comments="This is VM 2",
+                software_version=cls.software_versions[1],
             ),
             VirtualMachine.objects.create(
                 name="Virtual Machine 3",
@@ -350,6 +338,7 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
                 memory=3,
                 disk=3,
                 comments="This is VM 3",
+                software_version=cls.software_versions[2],
             ),
             VirtualMachine.objects.create(
                 name="Virtual Machine 4",
@@ -362,6 +351,7 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
                 memory=3,
                 disk=3,
                 comments="This is VM 4",
+                software_version=None,
             ),
             VirtualMachine.objects.create(
                 name="Virtual Machine 5",
@@ -374,6 +364,7 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
                 memory=3,
                 disk=3,
                 comments="This is VM 5",
+                software_version=None,
             ),
             VirtualMachine.objects.create(
                 name="Virtual Machine 6",
@@ -386,22 +377,28 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
                 memory=3,
                 disk=3,
                 comments="This is VM 6",
+                software_version=cls.software_versions[1],
             ),
         )
+        vms[0].software_image_files.set(cls.software_versions[1].software_image_files.all())
+        vms[1].software_image_files.set(cls.software_versions[0].software_image_files.all())
 
         int_status = Status.objects.get_for_model(VMInterface).first()
+        int_role = Role.objects.get_for_model(VMInterface).first()
         cls.interfaces = (
             VMInterface.objects.create(
                 virtual_machine=vms[0],
                 name="Interface 1",
                 mac_address="00-00-00-00-00-01",
                 status=int_status,
+                role=int_role,
             ),
             VMInterface.objects.create(
                 virtual_machine=vms[1],
                 name="Interface 2",
                 mac_address="00-00-00-00-00-02",
                 status=int_status,
+                role=int_role,
             ),
             VMInterface.objects.create(
                 virtual_machine=vms[2],
@@ -449,6 +446,15 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
         vms[0].tags.set(Tag.objects.get_for_model(VirtualMachine))
         vms[1].tags.set(Tag.objects.get_for_model(VirtualMachine)[:3])
 
+    def test_filters_generic(self):
+        # Assign more than 2 different software versions to VirtualMachine before we test generic filters
+        software_versions = list(SoftwareVersion.objects.all())
+        virtual_machines = list(VirtualMachine.objects.all())
+        for i in range(4):
+            virtual_machines[i].software_version = software_versions[i]
+            virtual_machines[i].save()
+        return super().test_filters_generic()
+
     def test_name(self):
         params = {"name": ["Virtual Machine 1", "Virtual Machine 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -457,6 +463,14 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
         params = {"comments": ["This is VM 1", "This is VM 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
+    def test_ip_addresses(self):
+        ipaddresses = list(IPAddress.objects.filter(vm_interfaces__isnull=False)[:2])
+        params = {"ip_addresses": [ipaddresses[0].address, ipaddresses[1].id]}
+        self.assertQuerysetEqualAndNotEmpty(
+            self.filterset(params, self.queryset).qs,
+            self.queryset.filter(interfaces__ip_addresses__in=ipaddresses).distinct(),
+        )
+
     def test_primary_ip4(self):
         params = {"primary_ip4": ["192.0.2.1/24", self.ipaddresses[0].pk]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
@@ -465,105 +479,6 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
         params = {"primary_ip6": ["fe80::8ef:3eff:fe4c:3895/24", self.ipaddresses[1].pk]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
-    def test_services(self):
-        params = {"services": [self.services[0].pk, self.services[1].name]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs,
-            self.queryset.filter(services__in=[self.services[0].pk, self.services[1].pk]),
-        )
-
-    def test_interfaces(self):
-        params = {"interfaces": [self.interfaces[0].pk, self.interfaces[1].pk]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs,
-            self.queryset.filter(interfaces__in=[self.interfaces[0].pk, self.interfaces[1].pk]),
-        )
-
-    def test_vcpus(self):
-        params = {"vcpus": [1, 2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_memory(self):
-        params = {"memory": [1, 2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_disk(self):
-        params = {"disk": [1, 2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_status(self):
-        params = {"status": [self.statuses[0].name, self.statuses[1].name]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs,
-            self.queryset.filter(status__in=[self.statuses[0], self.statuses[1]]),
-        )
-
-    def test_cluster_group(self):
-        groups = list(ClusterGroup.objects.all()[:2])
-        filter_params = [
-            {"cluster_group_id": [groups[0].pk, groups[1].pk]},
-            {"cluster_group": [groups[0].pk, groups[1].name]},
-        ]
-        for params in filter_params:
-            self.assertQuerysetEqualAndNotEmpty(
-                self.filterset(params, self.queryset).qs,
-                self.queryset.filter(cluster__cluster_group__in=groups).distinct(),
-            )
-
-    def test_cluster_type(self):
-        types = list(ClusterType.objects.all()[:2])
-        filter_params = [
-            {"cluster_type_id": [types[0].pk, types[1].pk]},
-            {"cluster_type": [types[0].pk, types[1].name]},
-        ]
-        for params in filter_params:
-            self.assertQuerysetEqualAndNotEmpty(
-                self.filterset(params, self.queryset).qs,
-                self.queryset.filter(cluster__cluster_type__in=types).distinct(),
-            )
-
-    def test_cluster(self):
-        clusters = Cluster.objects.all()[:2]
-        params = {"cluster_id": [clusters[0].pk, clusters[1].pk]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-        # 2.0 TODO: 'cluster' should match on name (This should be solved in FilterSet refactors)
-        # params = {'cluster': [clusters[0].name, clusters[1].name]}
-        # self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_location(self):
-        params = {"location": [self.locations[0].pk, self.locations[1].pk]}
-        self.assertQuerysetEqual(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(cluster__location__in=params["location"])
-        )
-        params = {"location": [self.locations[0].name, self.locations[1].name]}
-        self.assertQuerysetEqual(
-            self.filterset(params, self.queryset).qs,
-            self.queryset.filter(cluster__location__name__in=params["location"]),
-        )
-
-    def test_role(self):
-        roles = self.roles[:2]
-        params = {"role": [roles[0].pk, roles[1].name]}
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs, self.queryset.filter(role__in=[roles[0], roles[1]])
-        )
-
-    def test_platform(self):
-        platforms = self.platforms[:2]
-        filter_params = [
-            {"platform_id": [platforms[0].pk, platforms[1].pk]},
-            {"platform": [platforms[0].pk, platforms[1].name]},
-        ]
-        for params in filter_params:
-            self.assertQuerysetEqualAndNotEmpty(
-                self.filterset(params, self.queryset).qs,
-                self.queryset.filter(platform__in=[platforms[0], platforms[1]]).distinct(),
-            )
-
-    def test_mac_address(self):
-        params = {"mac_address": ["00-00-00-00-00-01", "00-00-00-00-00-02"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_has_primary_ip(self):
         params = {"has_primary_ip": "true"}
         self.assertQuerysetEqualAndNotEmpty(
@@ -588,11 +503,6 @@ class VirtualMachineTestCase(FilterTestCases.FilterTestCase, FilterTestCases.Ten
             self.queryset.filter(local_config_context_data__isnull=True),
         )
 
-    def test_search(self):
-        value = self.queryset.values_list("pk", flat=True)[0]
-        params = {"q": value}
-        self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
-
 
 class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
     queryset = VMInterface.objects.all()
@@ -608,6 +518,8 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
         ["mac_address"],
         ["mtu"],
         ["name"],
+        ("role", "role__id"),
+        ("role", "role__name"),
         ["parent_interface"],
         ["tagged_vlans", "tagged_vlans__pk"],
         ["tagged_vlans", "tagged_vlans__vid"],
@@ -639,8 +551,12 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
         )
 
         statuses = Status.objects.get_for_model(VMInterface)
+        roles = Role.objects.get_for_model(VMInterface)
+
+        vlans = VLAN.objects.filter()[:2]
+        vlans[0].locations.clear()
+        vlans[1].locations.clear()
 
-        vlans = VLAN.objects.filter(location=None)[:2]
         cls.vlan1 = vlans[0]
         cls.vlan2 = vlans[1]
 
@@ -651,6 +567,7 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
                 enabled=True,
                 mtu=100,
                 mac_address="00-00-00-00-00-01",
+                role=roles[0],
                 status=statuses[0],
                 description="This is a description of Interface1",
                 mode=InterfaceModeChoices.MODE_ACCESS,
@@ -662,6 +579,7 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
                 enabled=True,
                 mtu=200,
                 mac_address="00-00-00-00-00-02",
+                role=roles[1],
                 status=statuses[0],
                 description="This is a description of Interface2",
                 mode=InterfaceModeChoices.MODE_ACCESS,
@@ -687,6 +605,7 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
                 enabled=False,
                 mtu=300,
                 mac_address="00-00-00-00-00-04",
+                role=roles[0],
                 status=statuses[1],
                 description="This is a description of Interface4",
                 mode=InterfaceModeChoices.MODE_TAGGED,
@@ -699,6 +618,7 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
                 enabled=False,
                 mtu=300,
                 mac_address="00-00-00-00-00-05",
+                role=roles[1],
                 status=statuses[2],
                 description="This is a description of Interface5",
                 mode=InterfaceModeChoices.MODE_TAGGED_ALL,
@@ -745,12 +665,3 @@ class VMInterfaceTestCase(FilterTestCases.FilterTestCase):
             self.filterset(params, self.queryset).qs,
             self.queryset.filter(enabled=False),
         )
-
-    def test_search(self):
-        value = self.queryset.first().pk
-        params = {"q": value}
-        q = Q(id__iexact=str(value)) | Q(name__icontains=value)
-        self.assertQuerysetEqualAndNotEmpty(
-            self.filterset(params, self.queryset).qs,
-            self.queryset.filter(q),
-        )
